// Copyright 1994 A.D. Software. All Rights Reserved

// OOFTEST10

// this sample uses stream operators to set records

// Simple stream I/O is used to interact with the user.
#include "oofile.hpp"


DECLARE_CLASS(dbPeople)
	dbChar		LastName, OtherNames;
	dbDate		Birthday;
	dbShort		Short;
	dbUshort	Ushort;
	dbLong		Long;
	dbUlong		Ulong;
	dbReal		Real;
	dbText		Description;

	dbPeople() :
				dbTable("People"),
				LastName(39, "Last Name"),
				OtherNames(79, "Other Names"),
				Birthday("Birthday"),
				Short("Test Short"),
				Ushort("Test UShort"),
				Long("Test Long"),
				Ulong("Test ULong"),
				Real("Test Real"),
				Description("Description")
				{};
};

dbConnect_ctree theDB;
dbPeople	People;

int main()
{
	cout << "OOFILE Validation Suite - Test 10\n"
		 << "Simple test to demonstrate creating records" << endl
		 << "using input streams" << endl;
		 
	if (dbConnect::fileExists("ooftst10.db")) {
		theDB.openConnection("ooftst10.db");
		People.deleteAll();
	}
	else {
		theDB.newConnection("ooftst10.db");
	}

	dbDate::sDefaultDateOrder=dbDate::orderMDY;

// fake setting up an input stream, with known quantities.
// in normal life, we'd have an input stream straight from a file
// but we want predictable input so we can use the output for regression testing

	People.setSortOrder(People.OtherNames);
//	People.setReverseSortOrder(People.OtherNames);  shows known bug in reverse char sorts
	
	ostrstream oss;

	oss << "Dent" << '\t' << "Andy" << '\t' << "Jan 22nd 65"
		<< '\t' << -88 << '\t' << 21
		<< '\t' << -6500000 << '\t' << 7200000 << '\t' << 0.5
		<< '\t' << "Andy is the boss, and by the way none of the figures or dates in these test programs are real." 
		<< endl;
		
	oss << "Dent" << '\t' << "Trissa" << '\t' << "4-15-1971"
		<< '\t' << -7 << '\t' << 670
		<< '\t' << -19100000 << '\t' << 3400000 << '\t' << 1.81 
		<< '\t' << "Beyond Description" << endl;

	oss << "Dent" << '\t' << "Alice" << endl;
	oss << "Smith" << '\t' << "Billy" << endl;
		
	
	istream is(oss.rdbuf());  // normally something like ifstream is("filename");

	cout << "Entire database  - should be empty" << endl << theDB << endl;
	
	// change the following setting, or comment out this line
	// to see the error message from reading a short record
	People.setShortImports(true);
	
	is >> People;  // ********* Yes, this is the import!!! ********
	
	cout << "Entire database  - after importing from the stream" << endl << theDB << endl;
	
	cout << "done" << endl;

	return EXIT_SUCCESS;
}       