// COPYRIGHT 1995 A.D. Software, All rights reserved

// Private interface of PowerPlant Integration layer of OOFILE database

#ifndef H_OOFPowerPlantX
#define H_OOFPowerPlantX

#include "oofpp.hpp"
class dbView;

#include "LStrTableView.h"
#include <UTableHelpers.h>

class dbCharLink2EditField : public dbFieldLink {
public:
	dbCharLink2EditField(dbChar* fld, LEditField* ctl) :
						mField(fld), 
						mLinkTo(ctl)
						{};
	virtual void copyFieldToLink();
	virtual void copyLinkToField();
	
private:
	LEditField* mLinkTo;
	dbChar* mField;
};


class dbCharLink2TextEdit : public dbFieldLink {
public:
	dbCharLink2TextEdit(dbChar* fld, LTextEdit* ctl) :
						mField(fld), 
						mLinkTo(ctl)
						{};
	virtual void copyFieldToLink();
	virtual void copyLinkToField();

private:
	LTextEdit* mLinkTo;
	dbChar* mField;
};


class dbTextLink2TextEdit : public dbFieldLink {
public:
	dbTextLink2TextEdit(dbText* fld, LTextEdit* ctl) :
						mField(fld), 
						mLinkTo(ctl)
						{};
	virtual void copyFieldToLink();
	virtual void copyLinkToField();

private:
	LTextEdit* mLinkTo;
	dbText* mField;
};


class dbLongLink2EditField : public dbFieldLink {
public:
	dbLongLink2EditField(dbLong* fld, LEditField* ctl) :
						mField(fld), 
						mLinkTo(ctl)
						{};
	virtual void copyFieldToLink();
	virtual void copyLinkToField();
	
private:
	LEditField* mLinkTo;
	dbLong* mField;
};


class dbShortLink2EditField : public dbFieldLink {
public:
	dbShortLink2EditField(dbShort* fld, LEditField* ctl) :
						mField(fld), 
						mLinkTo(ctl)
						{};
	virtual void copyFieldToLink();
	virtual void copyLinkToField();
	
private:
	LEditField* mLinkTo;
	dbShort* mField;
};


class dbDateLink2EditField : public dbFieldLink {
public:
	dbDateLink2EditField(dbDate* fld, LEditField* ctl) :
						mField(fld), 
						mLinkTo(ctl)
						{};
	virtual void copyFieldToLink();
	virtual void copyLinkToField();
	
private:
	LEditField* mLinkTo;
	dbDate* mField;
};


class dbBrowserDisplay { // abstract base to allow us to wrap different controls
public:
	dbBrowserDisplay() {};
	virtual ~dbBrowserDisplay() {};
	virtual void loadData()=0;
	virtual void refresh()=0;
	virtual void changeViewTo(dbView*)=0;
	virtual long selectedRow() const=0;
};


class dbListBoxDisplay : public dbBrowserDisplay {
public:
	dbListBoxDisplay(LListBox* theLB, dbView* theView=0);
	virtual ~dbListBoxDisplay() {};
	virtual void loadData();
	virtual void refresh();
	virtual void changeViewTo(dbView*);
	virtual long selectedRow() const;
private:
	void DeleteAllData();
	void LoadRow(const int row, bool insertTheRow=false);
	void SetColWidths();
	
	LListBox* mDisplay;
	dbView* mView;
	bool mLoadedData;
};


class dbTableViewDisplay : public dbBrowserDisplay {
public:
	dbTableViewDisplay(LStrTableView* inTable, dbView* inView=0);
	virtual ~dbTableViewDisplay() {};
	virtual void loadData();
	virtual void refresh();
	virtual void changeViewTo(dbView*);
	virtual long selectedRow() const;
private:
	
	LStrTableView* mDisplay;
	LTableStorage* mStorage;  // created but not owned
	dbView* mView;
	bool mLoadedData;
};


class	dbViewAsTableStorage : public LTableStorage {
public:
						dbViewAsTableStorage(
								LTableView			*inTableView,
								dbView				*inView);
	virtual				~dbViewAsTableStorage() {};

	virtual void		SetCellData(
								const STableCell	&inCell,
								const void			*inDataPtr,
								Uint32				inDataSize);
	virtual void		GetCellData(
								const STableCell	&inCell,
								void				*outDataPtr,
								Uint32				&ioDataSize) const;
	virtual Boolean		FindCellData(
								STableCell			&outCell, 
								const void			*inDataPtr,
								Uint32				inDataSize) const;
								
	virtual void		InsertRows(
								Uint32				inHowMany,
								TableIndexT			inAfterRow,
								const void			*inDataPtr,
								Uint32				inDataSize);
	virtual void		InsertCols(
								Uint32				inHowMany,
								TableIndexT			inAfterCol,
								const void			*inDataPtr,
								Uint32				inDataSize);
								
	virtual void		RemoveRows(
								Uint32				inHowMany,
								TableIndexT			inFromRow);
	virtual void		RemoveCols(
								Uint32				inHowMany,
								TableIndexT			inFromCol);
									
	virtual void		GetStorageSize(
								TableIndexT			&outRows,
								TableIndexT			&outCols);

protected:
	dbView	*mView;  // ref only
};


#endif