// COPYRIGHT 1994 A.D. Software, All rights reserved

// public layer of OOFILE database


#include "oof2.hpp"
#include "oof3.hpp"
#include "oofrel.hpp"
#include "oofquery.hpp" 

#include <ctype.h>

#ifdef _Macintosh
// error reporting alert number
// if you have not included oofmac.rsrc or have changed the number
// of the alert you will need to fix the following
	#define kBigErrAlert  1001

	class macAlertMsg{
	public:
		macAlertMsg(const char*);
// use default copy ctor
	private:
		int FindWordBreakBefore(int bitLen, char* msg, int &nextWordStart);
	};
#endif


// define static variables
dbConnect* dbConnect::sCurrentlyConstructing;
dbTable* dbTable::sCurrentlyConstructing;
bool dbTable::sCloningWithoutSelection = false;
bool dbTable::sCloning = false;


bool 
skipTillDigit(istream& is)
{
	if (is.peek()=='\n')
		return false;  // early exit - empty line

	while (is.good()) {
	char c;
		is >> c;
		if ((c>='0') && (c<='9')) {
			is.putback(c);
			return true;
		}
	}
	return false;
}

// -------------------------------------------------------
//                      d b C o n n e c t
// -------------------------------------------------------
dbConnect::dbConnect() :
					mTables(4, 4),  /* go up in chunks of 4 tables at a time */ 
					mOpen(false)
{
	sCurrentlyConstructing = this;
}


dbConnect::~dbConnect() {
}


void 
dbConnect::attachTable(dbTable * theTable) 
{
	mTables.append(theTable);
	theTable->mBackend = MakeTableBackend(theTable);
}


dbTable *
dbConnect::table(unsigned int tableNum) const
{
	return (dbTable *) mTables.value(tableNum);
}


dbTable *
dbConnect::table(const char *tableName)
{
	return (dbTable *) mTables[tableName];	
}


void 
dbConnect::describe(ostream& os)
{
	os << "Connection:\t" << mConnectionName << endl;
	mTables.describe(os);
}


void 
dbConnect::dumpData(ostream& os)
{
	os << endl;
   unsigned int numTables = mTables.count();
	for (unsigned int i=0; i<numTables; i++)
	{
			dbTable  *theTable = (dbTable  *) mTables[i];  // safe downcast
			theTable->selectAll();
			os << "table:\t" << theTable->tableName() << endl; 
			theTable->extract(os);
			os << endl << endl << endl;  
	}
}


void 
dbConnect::generateTestData(const char* testFileName, unsigned long maxRecs)
{
	bool gotIt = false;
	char *buf = 0;
	unsigned int bufLen;
	// this is really silly, but I've got to do non-portable stuff
	// to get the length of the file, and read it into memory!
	
	ifstream fs(testFileName);
	if (fs) {
//		bufLen = filelength(fd);
//		bufLen = (bufLen > 10240) ? 10240 : bufLen;  // max 10kb
		bufLen = 10240;
		buf = new char[bufLen];
		assert(buf);
		fs.read(buf, bufLen);
		gotIt = (fs.gcount()==bufLen);
		fs.close();
	}
#ifdef oldSymantec_macintosh
	ifstream src(testFileName);
	if (src) {
		strstream inMemory;
		inMemory << src.rdbuf();
		src.close();
		buf = inMemory.str();
		bufLen = inMemory.rdbuf()->pcount();
		gotIt = true;
	}
#endif
	if (gotIt) {
   	unsigned int numTables = mTables.count();
		for (unsigned int i=0; i<numTables; i++)
		{
				unsigned long maxRecsForTable = maxRecs ? maxRecs : rand()%5000;   // specified or random up to 5,000
				dbTable  *theTable = (dbTable  *) mTables[i];  // safe downcast
				theTable->generateTestData(maxRecsForTable, buf, bufLen);  // up to 5000 recs
		}

		delete[] buf;
	}
}


bool 
dbConnect::fileExists(const char * fName)
{
	FILE *fp;

	fp = fopen(fName, "rb" );
	if (fp) {
		fclose( fp );
		return true;
	}
	else
		return false;
}


void 
dbConnect::raise(ostream& os)
{
#ifdef _Windows
	strstream oss;
	oss << os.rdbuf() << ends;
	char* s = oss.str();
	::MessageBox(0, s, "OOFILE Error", MB_ICONSTOP);
	oss.rdbuf()->freeze(0);  // leave buffer for stream to delete
#else
	#ifdef _Macintosh
		ostrstream oss;
		#ifdef __MWERKS__
			oss << *(os.rdbuf()) << ends;   // they don't define operator<<(streambuf*)
		#else
			oss << os.rdbuf() << ends;
		#endif
		char* s = oss.str();
		macAlertMsg x(s);
		oss.rdbuf()->freeze(0);  // leave buffer for stream to delete
	#else
		cout << os.rdbuf();
		cout << "Press Return to continue";
		char c;
		cin.flags(cin.flags() & ~ios::skipws);
		cin >> c;
	#endif
#endif
	abort();
}


void 
dbConnect::raise(const char* str)
{
	#ifdef _Windows
		::MessageBox(0, str, "OOFILE Error", MB_ICONSTOP);
	#else
		#ifdef _Macintosh
			macAlertMsg x(str);
		#else		
			cout << str << endl;
			cout << "Press Return to continue";
			char c;
			cin.flags(cin.flags() & ~ios::skipws);
			cin >> c;
		#endif
	#endif
	abort();
}


#ifdef _Macintosh
macAlertMsg::macAlertMsg(const char* constMsg)
{
	Str255 s0, s1, s2, s3;
	s0[0] = s1[0] = s2[0] = s3[0] = 0;
	int lenToNextStart, lenToWordBreak, bitLen;
	
	char* msg = (char*) constMsg;  // cast away const as some of the toolbox routines aren't const
	                               // but *are* safe, so we are preserving the intent
	long msgLen = strlen(msg);

	if (msgLen>255) {
		lenToWordBreak = FindWordBreakBefore(255, msg, lenToNextStart);
		msgLen -= (lenToNextStart+1);
	}
	else {
		lenToWordBreak = msgLen;
		msgLen = 0;
	}
		
	memcpy(&s0[1], msg, lenToWordBreak);
	s0[0] = lenToWordBreak;
	
	if (msgLen>0) {
		msg += lenToNextStart; 
		bitLen = (msgLen>255) ? 255 : msgLen+1;  // point to nil byte at end of string
		lenToWordBreak = FindWordBreakBefore(bitLen, msg, lenToNextStart);
		memcpy(&s1[1], msg, lenToWordBreak);
		s1[0] = lenToWordBreak;
		msgLen -= (lenToNextStart+1);
	}
	
	if (msgLen>0) {
		msg += lenToNextStart; 
		bitLen = (msgLen>255) ? 255 : msgLen+1;
		lenToWordBreak = FindWordBreakBefore(bitLen, msg, lenToNextStart);
		memcpy(&s2[1], msg, lenToWordBreak);
		s2[0] = lenToWordBreak;
		msgLen -= (lenToNextStart+1);
	}
	
	if (msgLen>0) {
		msg += lenToNextStart; 
		bitLen = (msgLen>255) ? 255 : msgLen+1;
		lenToWordBreak = FindWordBreakBefore(bitLen, msg, lenToNextStart);
		memcpy(&s3[1], msg, lenToWordBreak);
		s3[0] = (lenToNextStart+1);
	}
	
	ParamText(s0, s1, s2, s3);
	StopAlert(kBigErrAlert, nil);
}


int 
macAlertMsg::FindWordBreakBefore(int bitLen, char* msg, int &lenToNextStart)
{
	char* scanFrom = msg+bitLen;
	lenToNextStart = bitLen;   // as far as we know - we aren't allowed to skip right
	char* endWord = scanFrom;  // in case all one word
	if (isspace(*scanFrom)) {  // skip blanks we started on
		while ((scanFrom>msg) && isspace(*scanFrom))
			scanFrom--;
	}
	else {
		if (*scanFrom!='\0') { // not pointing at terminating null
			while ((scanFrom>msg) && !isspace(*scanFrom))
				scanFrom--;
			if ((scanFrom>msg)) {
				char* splitWordStart = scanFrom+1;
				while (isspace(*scanFrom) && (scanFrom>msg))  // skip blanks we reached
					scanFrom--;
				if ((scanFrom>msg))
					lenToNextStart = splitWordStart-msg+2;
			}
		}
	}
	return scanFrom-msg+1;		
}
#endif



// -------------------------------------------------------
//                      d b T a b l e
// -------------------------------------------------------
dbTable::dbTable(const char *tableName) :
								mTableName(tableName),
								mFieldCount(0),
								mSaveOption(requireExplicit),
								mAllowShortImports(false),
								mRelated(0)
{
#ifdef OOF_Debug
	if(dbConnect::sCurrentlyConstructing==0)
		dbConnect::raise(ostrstream() << "dbTable: " << tableName 
		<< " is defined AFTER the call to newConnection or openConnection, too late to be part of this database connection's schema");   
#endif
	dbTable::sCurrentlyConstructing = this;
	dbConnect::sCurrentlyConstructing->attachTable(this);
}



dbTable::~dbTable()  
{
	delete mBackend;
}


dbTable::dbTable(const dbTable& rhs) :
				mFields(rhs.mFieldCount),  // will be populated by dbField copy ctor attaching to table
				mTableName(rhs.mTableName),
				mFieldCount(0),
				mSaveOption(rhs.mSaveOption),
				mAllowShortImports(rhs.mAllowShortImports),
				mRelated(0)  // is set when the relationship is instantiated
{
	mBackend = rhs.mBackend->clone(sCloningWithoutSelection, this);	
	sCloningWithoutSelection = false;
	sCurrentlyConstructing = this;    // so fields that are about to invoke dbField copy constructor point to ME
	if (!sCloning)  // called from user copy ctor, not via clone()
		new dbCloneCleanup(this);  // self-destroying dependant
}


void 
dbTable::buildSchema() 
{
	if (mTableName.isEmpty()) {
		mTableName = defaultName();
	}
	mBackend->buildSchema();
}


void
dbTable::validateContext()
{
	if (!isRecordLoaded()) {
		if (mRelated)   // this is a clone managing a related selection
			mRelated->relateFromRecord();
		else
			start();
	}
}


void
dbTable::ValidatePossiblyRelatedContext()
{
	if (!isRecordLoaded()) 
		if (mRelated)   // this is a clone managing a related selection
			mRelated->relateFromRecord();
} 


void
dbTable::LoadRelatedContext()
{
	assert(mRelated);   // this is a clone managing a related selection
	mRelated->relateFromRecord();
}


ostream& 
operator<<(ostream& os, dbTable& tbl)
{
	tbl.extract(os);
	return os;
}


ostream& 
operator<<(ostream& os, dbTable* tbl)
{
	tbl->extract(os);
	return os;
}


istream& 
operator>>(istream& is, dbTable* tbl)
{
	tbl->insert(is);
	return is;
}


istream& 
operator>>(istream& is, dbTable& tbl)
{
	tbl.insert(is);
	return is;
}


bool 
dbTable::pointsToBackend(const OOF_tableBackend* rhsBackend) const
{
	return mBackend==rhsBackend;
} 


bool 
dbTable::canSaveRecord()
{
	return true;
}


dbTable* 
dbTable::cloneWithoutSelection()
{
	sCloningWithoutSelection = true;
	dbTable* ret = new dbTable(*this);
	assert(ret);
	ret->postCloneCleanup();
	return ret;
}


void 
dbTable::postCloneCleanup()
{
	unsigned int numFields = mFields.count();
	for (unsigned int i=0; i<numFields; i++)
	{
		dbField  *theField = field(i);
		theField->postCloneCleanup();
	}
	sCloning = false;
}


void 
dbTable::attachfield(dbField * theField)  {
	mFields.append(theField);
	CompleteField(theField);
}


void 
dbTable::extract(ostream& os)
{
// NOTE: mFields.table = the database/selection
// mFields = ordered list of fields to report
	
	start();		// start record iteration (vertical)
	unsigned int numFields = mFields.count();
	unsigned long numRecs = count();
	for (unsigned long row=0; row<numRecs; row++) {
		for (unsigned int i=0; i<numFields; i++)
		{
				dbField  *theField = (dbField  *) mFields[i];  // safe downcast
				os << theField << "\t";
		}
		os << endl;
		if (row<numRecs-1)
			next();
	}
}


bool 
dbTable:: allowsShortImports() const
{
	return mAllowShortImports;
}


void 
dbTable:: setShortImports(bool allowShorties)
{
	mAllowShortImports = allowShorties;
}

 
unsigned long 
dbTable::insert(istream& is)
{
// later will have a dbView::insert() to let us just import into specified fields

	suspendSorting();  // otherwise will resort after every record read!
	selectAll();  // prevents us incurring the overhead of expanding a selection
	
	const char kFieldSep = '\t'; // later allow user setting of divider chars
	const char kRecSep = '\n';
	
	unsigned long numAdded = 0;
	is.flags(is.flags() & ~ios::skipws);  // we look at the white space!
	while (is.good()) {
		bool gotCompleteRecord = false;
		newRecord();		// start record iteration (vertical)
		unsigned int numFields = mFields.count();
		for (unsigned int i=0; i<numFields; i++)
		{
			dbField  *theField = (dbField  *) mFields[i];  // safe downcast
			if (!theField->insert(is, kFieldSep, kRecSep))  
				break;  // field failed for some reason, probably partial record
				
			if (is.good()) { // should be at field or rec separator, NOT end of stream
				char c;
				is >> c;  // consume record or field separator
				if (c!=kFieldSep) { // most likely is FieldSep, which just loops around
					assert(c==kRecSep);  // otherwise should be end of record
					unsigned int expectedFields = (numFields-1);
					if (i==expectedFields)
						gotCompleteRecord = true;
					else 
						if (mAllowShortImports)
							gotCompleteRecord = true;
						else {
							gotCompleteRecord = false;
#ifdef OOF_Debug
		dbConnect::raise(ostrstream() << "dbTable::insert() in table: " << tableName()
		                              << " has hit a short record of " << i << " out of "
																	<< expectedFields << " expected fields" 
																	<< " on import record " << numAdded
										);   
#endif
					}
					break;  // end of record, short or complete
				}
			}  // stream is good
			else {
				if (is.eof())
					break;  // end of file exit for g++ streams lib peculiarity
					
#ifdef OOF_Debug
				dbConnect::raise(ostrstream() << "dbTable::insert() in table: " << tableName() 
				                              << " has a problem with import record " << numAdded
												);   
#endif
				break;  // error break - problem with stream
			}
		}  // loop through fields
		
		if (gotCompleteRecord) { 		
			numAdded++;
			saveRecord();
			unloadRecord();
		}
		else {
			unloadRecord();  // unload last probably partial record
			break;  // ABNORMAL EXIT FROM LOOP
		}
	} // while getting recs
	resumeSorting();
	broadcast(OOFmsg_ChangeSelection);
	return numAdded;
}


void 
dbTable::saveRecord()
{
	do {
	    if (mRelated)  // this is a clone on the rhs of a relationship
			mRelated->updateRelField();  // so save the relationship
// WARNING this seems like a mistake, that this is an operation that should be performed
// once only. Remember that this copying only occurs for the rhs of a relationship which
// would often be a selection of multiple records.
		mBackend->saveRecord();
		broadcast(OOFmsg_SaveRecord);  // AFTER saving us, so our oid is updated for any dependents
	}
	while (isDirty());  // one of our dependents may have dirtied us
}


void 
dbTable::deleteRecord()
{
	if (isRecordLoaded()) {
		relateAllDeletables();
		broadcast(OOFmsg_DeleteRecord);
		mBackend->deleteRecord();
	}
}


void 
dbTable::deleteSelection()
{
	unsigned long numRecs = count();
	for (unsigned long i=0; i<numRecs; i++)
	{
		start();	
		deleteRecord();
	}
	selectNone();
}


void 
dbTable::deleteAll()
{
	selectAll();
	deleteSelection();
}


void 
dbTable::describe(ostream& os)
{
	if (mTableName.isEmpty())
		os << "Un-named table";
	else
		os << "table: " << mTableName;
	os << " contains fields:" << endl;
	mFields.describe(os);
}


void 
dbTable::generateTestData(unsigned long numRecords, const char* testBuf, unsigned long testBufLen)
{
    unsigned int numFields = mFields.count();
	 for (unsigned int r=0;r<numRecords; r++) {
	 	newRecord();
		for (unsigned int i=0; i<numFields; i++)
		{
				dbField  *theField = (dbField  *) mFields[i];  // safe downcast
				theField->generateTestData(testBuf, testBufLen); 
		}
		saveRecord();
	}
}


void 
dbTable::start() 
{
	ValidatePossiblyRelatedContext();
	if (atFirst())
		return;  // optimisation, going to current record
		
	ContextChange();
	mBackend->start();
}


void 
dbTable:: newRecord()
{
	ContextChange();
	mBackend->newRecord();
//	broadcast(OOFmsg_NewRecord);  // can't think of reason why now, maybe to support GUI
}


void 
dbTable::ContextChange()
{
	switch (mSaveOption) {
	case autoOnContextChange :
		if (isRecordLoaded()) {
            saveRecord();
        }
		break;
	
	case requireExplicitAndBuffer :
		broadcast(OOFmsg_ChangeContext);  // in the other cases we broadcast messages like msgSaveRecord
		mBackend->CachingContextChange();
		break;
		
	case requireExplicit :
#ifdef OOF_Debug
		if (isDirty())
			dbConnect::raise(ostrstream()  << "ContextChange called on dirty record without saving: " 
			<< "file name: " << tableName() 
			<< endl);
#endif
		unloadRecord();
		break;	
		
	default :
		assert(0);
	}	
}



bool 
dbTable::gotoRecord(unsigned long relativeNum)
{
	ValidatePossiblyRelatedContext();
	if (mBackend->atRelative(relativeNum))
		return true;  // optimisation, going to current record
	ContextChange();
	return mBackend->gotoRecord(relativeNum);
}


bool 
dbTable::loadRelatedContextJoiningFromTo(const dbField* f1, const dbField* f2)
{
	broadcast(OOFmsg_UnloadRecord);
// bit of redirection here - as a destination table, almost certainly in a relation,
// the field to passed in is probably a "real" field pointing back to the original table
// not the cloned destination of a relation. To make sure no side effects can change
// the original table, retrieve our copy of the field.
	dbField* reallyTo = (dbField*) mFields[f2->fieldNumber()];  // safe downcast
	
	return mBackend->loadRelatedContextJoiningFromTo(f1, reallyTo);
}


void
dbTable::relateAllDeletables()
{
	unsigned short numFields = mFields.count();
	for (unsigned short i=0; i<numFields; i++) {
		dbField* fld = (dbField*) mFields[i];  // safe downcast
		if (fld->fieldType()==relationshipField) {
			dbRelRefBase* rel = (dbRelRefBase*) fld;  // safe downcast
			if (rel->propagatesDeletes()) {
// just because the traversal field exists, doesn't mean we have a fully instantiated
// relationship - we need to refer to the rhs table and we shouldn't bother relating
// if there's no related records!
                dbTable* relProto = rel->relatedTable(); 
                assert(relProto);
                relProto->LoadRelatedContext();
            }
		}
	}
}


/*
bool 
dbTable::searchDefaultIndex(const char* defaultIndexSearchStr)
{
	dbQueryBinary Q(new dbQueryField(field(mSortByFieldNum)), dbQueryClause::equals, 
		new dbQueryLiteralStr(defaultIndexSearchStr));
	return search(Q);
}
*/

bool 
dbTable::search(const dbQueryClause& query)
{
	suspendSorting();  // don't want to trigger a sort
	ContextChange();
	bool foundAny = mBackend->search(&query);
	resumeSorting();
	broadcast(OOFmsg_ChangeSelection);
	return foundAny;
}


bool 
dbTable::searchSelection(const dbQueryClause& query)
{
	suspendSorting();  // don't want to trigger a sort
	ContextChange();
	bool foundAny = mBackend->searchSelection(&query);
	resumeSorting();
	broadcast(OOFmsg_ChangeSelection);
	return foundAny;
}

bool 
dbTable::searchSelContainsAnyDelimited(const dbField& schField, const char *schStr, char delim)
{
	return mBackend->searchSelContainsAnyDelimited(&schField, schStr, delim);
}

bool
dbTable::searchSelContainsAnyOf(const dbField& schField, const char **schStrs, unsigned long count)
{
	return mBackend->searchSelContainsAnyOf(&schField, schStrs, count);
}

bool 
dbTable::searchSelContainsAllDelimited(const dbField& schField, const char *schStr, char delim)
{
	return mBackend->searchSelContainsAllDelimited(&schField, schStr, delim);
}

bool
dbTable::searchSelContainsAllOf(const dbField& schField, const char **schStrs, unsigned long count)
{
	return mBackend->searchSelContainsAllOf(&schField, schStrs, count);
}
	

void 
dbTable::unSorted()
{
	mBackend->setSortOrder(0);
}
	

void 
dbTable::setSortOrder(const dbField& theField)
{
    dbSorter* currentSorter = sorter();
    if (currentSorter && currentSorter->sortsBySingleField())
        ((dbFieldSorter *)currentSorter)->changeSortFieldTo(theField.fieldNumber());  // safe downcast
    else
        mBackend->setSortOrder(new dbFieldSorter(this, mBackend, theField.fieldNumber()));
}


void 
dbTable::setReverseSortOrder(const dbField& theField)
{
   setSortOrder(theField);
   ((dbFieldSorter*)sorter())->setReverseSort();  // safe downcast
} 


void 
dbTable::suspendSorting() const
{
    dbSorter* currentSorter = sorter();
	if (currentSorter)
		currentSorter->suspendSorting();
} 


void 
dbTable::resumeSorting() const
{
    dbSorter* currentSorter = sorter();
	if (currentSorter)
		currentSorter->resumeSorting();
} 


void 
dbTable::unloadRecord()
{
	broadcast(OOFmsg_UnloadRecord);
	mBackend->unloadRecord();
}


void 
dbTable::CompleteField(dbField * theField)
{
	theField->mBackend = mBackend;
	theField->mFieldNumber = mFieldCount++;
	theField->mTable = this;
}


// -------------------------------------------------------
//           O O F _ F i e l d S e g O w n e r
// -------------------------------------------------------
OOF_FieldSegOwner::OOF_FieldSegOwner()
{
	mSegments.ownsContents();
}


OOF_FieldSegOwner::OOF_FieldSegOwner(const OOF_FieldSegOwner& rhs) :
	mSegments(rhs.mSegments)  // ref counted copy	
{
}


fieldNumT 
OOF_FieldSegOwner::fieldNumber(unsigned int i) const
{
	return segment(i)->mFieldNo;
} 


void 
OOF_FieldSegOwner::addSegment(dbField& segField) {
	OOF_Segment* seg = new OOF_Segment(segField);
	mSegments.append(seg);
}


OOF_FieldSegOwner& 
OOF_FieldSegOwner::operator>>(dbField& segField)
{
	addSegment(segField);
	segment(countSegments()-1)->setReverse();
	return *this;
}


// -------------------------------------------------------
//                      d b S o r t e r 
// -------------------------------------------------------
dbSorter::dbSorter(dbTable* listensTo, OOF_tableBackend* backend) : 
    dbSingleListener(listensTo),
	mTable(listensTo),
    mSorts(backend),
    mSortOnSaves(true),
	mNeedsToSort(true)
{}


dbSorter::dbSorter(const dbSorter& rhs, dbTable* listensTo, OOF_tableBackend* backend) : 
    dbSingleListener(0),  // this is NOT a simple copy as we redirect listener
	OOF_FieldSegOwner(rhs),
	mTable(listensTo),
    mSorts(backend),
    mSortOnSaves(rhs.mSortOnSaves),
	mNeedsToSort(rhs.mNeedsToSort)
{
	mSuspendCount = rhs.mSuspendCount;
	
	if (rhs.mSuspendedListensTo) { 
		assert(mSuspendCount);
		mSuspendedListensTo = listensTo;   // assume is suspended but still different broadcaster
		mListensTo = 0;
	}
	else {
		assert(mSuspendCount==0);
		// assert(listensTo);  // should never be called without a broadcaster
		if (listensTo)
			subscribeTo(listensTo);
		mSuspendedListensTo = 0;
	}
} 


dbSorter* 
dbSorter::clone(dbTable* listensTo, OOF_tableBackend* backend) const 
{ 
    dbSorter* ret = new dbSorter(*this, listensTo, backend);
    return ret;
}


dbSorter& 
dbSorter::operator>>(dbField& segField)
{
	addSegment(segField);
	segment(countSegments()-1)->setReverse();
	return *this;
} 


void 
dbSorter::sortNow()
{ 
   assert(mSorts);
   mNeedsToSort = false;
   mSorts->sortSelectionNow(this);  // NOTE no reverse case
}


unsigned long 
dbSorter::indexNo() const
{
    return 0;
}


const dbField* 
dbSorter::field(unsigned int n) const
{
	const dbField* theField = mTable->field(OOF_FieldSegOwner::fieldNumber(n));
	return theField;
}


void 
dbSorter::receiveMsg(OOFmsgT msg, unsigned long senderDefined)
{
	switch (msg) {
	case (OOFmsg_UnloadRecord) :
	case (OOFmsg_ChangeContext) :
		if (mNeedsToSort)
			sortNow();  
		break;

	case (OOFmsg_ChangeSelection) :
		mNeedsToSort = true;  // don't assume we should sort now, they may not need the selection  
		break;

	case (OOFmsg_SaveRecord) :
		if (mSortOnSaves)
 		   sortNow();  
      break;
      
   default :
		dbSingleListener::receiveMsg(msg, senderDefined);
	};
}

void 
dbSorter::suspendSorting()
{
	suspendListening();
} 


void 
dbSorter::resumeSorting()
{
	resumeListening();
} 


unsigned short
dbSorter::sortFieldNo() const
{
      return 0;
}


bool
dbSorter::sortsBySingleField() const
{
      return false;
}
 
 

// -------------------------------------------------------
//                      d b F i e l d S o r t e r 
// -------------------------------------------------------
dbFieldSorter::dbFieldSorter(dbTable* listensTo, OOF_tableBackend* backend, fieldNumT sortField) : 
    dbSorter(listensTo, backend),
    mSortFieldNo(sortField),
    mReverseSort(false)
{}
 
 
dbSorter* 
dbFieldSorter::clone(dbTable* listensTo, OOF_tableBackend* backend) const
{
    dbFieldSorter* ret = new dbFieldSorter(listensTo, backend, mSortFieldNo);
    ret->setSortOnSaves(mSortOnSaves);
    ret->setReverseSort(mReverseSort);
    return ret;
}


unsigned short
dbFieldSorter::sortFieldNo() const
{
      return mSortFieldNo;
}


bool
dbFieldSorter::sortsBySingleField() const
{
      return true;
}
 
 
void
dbFieldSorter::sortNow()
{
   assert(mSorts);
   mNeedsToSort = false;
   if (mReverseSort)
      mSorts->sortSelectionInverseNow(mSortFieldNo);
   else
      mSorts->sortSelectionNow(mSortFieldNo);
}

void
dbFieldSorter::changeSortFieldTo(fieldNumT  newNum)
{
   if (mSortFieldNo==newNum)
      return;  // trivial case
   
   mSortFieldNo = newNum;
   mNeedsToSort = true;
   resumeListening();  // in case we're suspended
}


// -------------------------------------------------------
//                  r e v e r s e 
// -------------------------------------------------------
// functor used instead of dbSorter::operator>> for people
// who prefer
// dbSorter() << field1 << reverse(field2);
// to
// dbSorter() << field1 >> field2

// NOTE the following 2 functions are prototyped by their friend declarations in
// class reverse
dbSorter& 
operator<<(dbSorter& os, const reverse& r)
{
	os >> r.mField;
	return os;
}


OOF_FieldSegOwner& 
operator<<(OOF_FieldSegOwner& os, const reverse& r)
{
	os >> r.mField;
	return os;
}
 