#ifndef H_OOF1
#define H_OOF1

// COPYRIGHT 1994 A.D. Software, All rights reserved

// public layer of OOFILE database

// NOTE inline definitions included at end of this header file

// see misc.txt for code conventions
#include "oof0.hpp"
#include "oofmsg.hpp"
#ifdef _Macintosh
#include <Types.h>
#endif


typedef unsigned long oidT;
enum OOF_IndexOptions { kNotIndexed=0, kIndexed=1, kIndexNoDups=2, kIndexCaseSensitive=4,
												kIndexCompressLeading=8, kIndexCompressPadding=16,
												kIndexCompress = kIndexCompressLeading + kIndexCompressPadding,
												kIndexCompressNoDups = kIndexCompress + kIndexNoDups,
												kIndexCompressLeadingNoDups = kIndexCompressLeading + kIndexNoDups,
												kIndexCompressPaddingNoDups = kIndexCompressPadding + kIndexNoDups
											}; 
/* NEW VERSION
enum OOF_IndexOptions { kNotIndexed=0, kIndexed=1, kIndexedNoDups=2, kIndexedCaseSensitive=4,
												kIndexedCompressLeading=8, kIndexedCompressPadding=16,
												kIndexedCompress = kIndexedCompressLeading + kIndexedCompressPadding,
												kIndexedCompressNoDups = kIndexedCompress + kIndexedNoDups,
												kIndexedCompressLeadingNoDups = kIndexedCompressLeading + kIndexedNoDups,
												kIndexedCompressPaddingNoDups = kIndexedCompressPadding + kIndexedNoDups
											};
											*/
// for backwards compatability	and to balance things out so we have both a verb and noun form
// (the original one is as follows except for kNotIndexed and kIndexed
/*enum OOF_IndexOptions2 { kNotIndex=0, kIndex=1, kIndexNoDups=2, kIndexCaseSensitive=4,
												kIndexCompressLeading=8, kIndexCompressPadding=16,
												kIndexCompress = kIndexCompressLeading + kIndexCompressPadding,
												kIndexCompressNoDups = kIndexCompress + kIndexNoDups,
												kIndexCompressLeadingNoDups = kIndexCompressLeading + kIndexNoDups,
												kIndexCompressPaddingNoDups = kIndexCompressPadding + kIndexNoDups
											};
											*/
ostream& operator<<(ostream&, OOF_IndexOptions);


typedef  unsigned short fieldNumT;  // never more than 64k fields in a record!

// utility functions
bool skipTillDigit(istream&);


class OOF_String {
public:
// constructors
	OOF_String(const OOF_String&);
	OOF_String(const OOF_String&, const char *);
	OOF_String(const OOF_String&, const char, const char *);
	OOF_String(const OOF_String&, const char *, const OOF_String&);
	OOF_String(const char *chars=0);  // default
	~OOF_String();

// other protocol
	OOF_String& operator=(const OOF_String&);
	OOF_String& operator=(const char*);
	operator char *() const; 
	char *adopt();
	bool isEmpty() const;
	
protected:
// data storage
	char *mBody;	// owned
	unsigned int mLen;
	
public:
 static char sEmptyStr;	
};
ostream& operator<<(ostream&, const OOF_String&);


#ifdef _Macintosh
class OOF_MacString : public OOF_String {
public:
	OOF_MacString(const Str63);
// use default copy ctor
};
#endif



class dbQueryClause;
class dbRelRefBase;
class dbField;
class OOF_tableBackend;
class dbContext;
class dbConnect;
class OOF_ExpandableLongArray;
class dbSorter;

class dbTable : public dbClass, public dbBroadcaster {
public:
	dbTable(const char *name=0);  
	dbTable(const dbTable&);
	virtual ~dbTable();
private:
	void operator=(const dbTable&) {assert(0);};
public:
	virtual const char* defaultName() const { return "\0"; };
	void setName(const char*);
	virtual dbTable* cloneWithoutSelection();
	void postCloneCleanup();
//	operator dbTable*();

	void validateContext();
	void relateAllDeletables();
	
	// hiding oid's which may be record offsets or generated unique numbers
	oidT currentOID();
	
// other protocol
	bool loadRelatedContextJoiningFromTo(const dbField*, const dbField*);
	enum saveOptions{requireExplicit, autoOnContextChange, requireExplicitAndBuffer};
	void setSaveOption(const saveOptions);
	dbField *field(fieldNumT) const;
	dbField *field(const char *);
	const OOF_String& tableName() const;
	void setOIDfield(const dbField&) const;
	bool pointsToBackend(const OOF_tableBackend*) const;
	virtual bool canSaveRecord();  // expect user override in many cases

	void attachfield(dbField*);
	void buildSchema();
	void createTableInConnection(const dbConnect*);
	void openTableInConnection(const dbConnect*);
	unsigned int numIndexes() const;
	unsigned int numFiles() const;
	long sequenceNumber() const;
		
// recordwise access
	void start();
	void next();
	void first();
	void prev();
 	void last();

	bool more() const;
	bool atFirst() const;
	bool atLast() const;
	bool atRelative(unsigned long) const;

	unsigned long count();
	unsigned long countAll();

// sorting	
	void setSortOrder(const dbField&);
	void setSortOrder(const dbSorter&);
	void setReverseSortOrder(const dbField&);
	dbSorter* sorter() const;
	void unSorted();
	void suspendSorting() const;
	void resumeSorting() const;

	void unloadRecord();
	bool isRecordLoaded() const;
	bool isDirty() const;

// public search functions
    virtual unsigned long recordNumber() const;
	bool gotoRecord(unsigned long);
//	bool searchDefaultIndex(const char*);
	void selectAll();
	void selectNone();
	bool search(const dbQueryClause& query);
	bool searchSelection(const dbQueryClause& query);
	bool searchSelContainsAnyDelimited(const dbField& schField, const char *schStr, char delim);
	bool searchSelContainsAnyOf(const dbField& schField, const char **schStrs, unsigned long count);
	bool searchSelContainsAllDelimited(const dbField& schField, const char *schStr, char delim);
	bool searchSelContainsAllOf(const dbField& schField, const char **schStrs, unsigned long count);
	
// selection functions
	void difference_with(const dbTable&);
	void operator -=(const dbTable&);
	void operator %=(const dbTable&);
	
	void intersection_with(const dbTable&); 
	void operator &=(const dbTable&);
	
	void union_with(const dbTable&);
	void operator +=(const dbTable&);
	void operator |=(const dbTable&);
	
	void invert();
	void operator~();
	void operator!();
	
// selection of specific contexts
	void selectJustRecord(unsigned long recNo);
	void selectJustOID(oidT);
	void selectJustCurrent();
	
// data access
	void newRecord();
	void saveRecord();
	void deleteRecord();
	void deleteSelection();
	void deleteAll();
	
	void extract(ostream&);
	unsigned long insert(istream&);
	bool allowsShortImports() const;
	void setShortImports(bool);

// reflective operators
	virtual void describe(ostream&);

	virtual void generateTestData(unsigned long numRecords, const char* testBuf, unsigned long testBufLen);

private:
	void CompleteField(dbField*);
	void ContextChange();
	void ValidatePossiblyRelatedContext();
	void LoadRelatedContext();

// data storage
protected:
	OOF_tableBackend *mBackend;  	// owned  (the bit that does all the work!!)
	OOF_Dictionary	mFields;
	OOF_String mTableName;
	fieldNumT mFieldCount;
	saveOptions mSaveOption;
	bool mAllowShortImports;
	
private:
	dbRelRefBase* mRelated;  // if we are a clone managing a related selection, this is how we get back
	
	static dbTable *sCurrentlyConstructing;
protected:
	static bool sCloningWithoutSelection;
	static bool sCloning;
	
	friend class dbField;
	friend class dbConnect;
	friend class dbRelHalf;
	friend class dbRelRefBase;
	friend class OOF_tableBackend;
};
ostream& operator<<(ostream& os, dbTable& tbl);
ostream& operator<<(ostream& os, dbTable* tbl);
istream& operator>>(istream& is, dbTable* tbl);
istream& operator>>(istream& is, dbTable& tbl);


class dbConnect : public dbClass {
public:
	dbConnect();
	virtual ~dbConnect();
private:
	dbConnect(const dbConnect&) { assert(0); };  // NEVER CALLED

public:	
	virtual void newConnection(const char* connectionName)=0;
	virtual void openConnection(const char* connectionName)=0;
	virtual void close()=0;

	void attachTable(dbTable *);
	dbTable *table(unsigned int) const;
	dbTable *table(const char*);

// locking
	virtual void exitLocking()=0;
	virtual void enterWriteLocking()=0;
	virtual void enterReadLocking()=0;

// utilities
	static bool fileExists(const char * fName);
	static bool underConstruction();
	virtual void describe(ostream&);
	void dumpData(ostream&);
	void generateTestData(const char* testFileName, unsigned long maxRecs=0);
	static void raise(ostream&);
	static void raise(const char*);

private:
	virtual OOF_tableBackend *MakeTableBackend(dbTable*)=0;

// data storage
protected:
	static dbConnect *sCurrentlyConstructing;
	OOF_Dictionary	mTables;
	OOF_String mConnectionName;
	bool mOpen;	
	
	friend dbTable::dbTable(const char*);
	friend class dbRelationship;
};


class dbBLOB;

class OOF_tableBackend {
// uses lots of public methods but is only ever accessed by a dbTable or dbField subclass
// so no problems
protected:
	OOF_tableBackend(dbTable* theTable);
	OOF_tableBackend(const OOF_tableBackend&, dbTable*);
	virtual ~OOF_tableBackend() {};
	
 public:
	virtual void buildSchema()=0;
	virtual void createTableInConnection(const dbConnect*)=0;
	virtual void openTableInConnection(const dbConnect*)=0;
	void setOIDfield(const dbField &oidfield);
	virtual OOF_tableBackend* clone(bool selectionNotCopied, dbTable*) const=0;

// OID access & generation
	virtual oidT currentOID()=0;

// recordwise access
	virtual void start()=0;
	virtual void next()=0;
	virtual void prev()=0;
	virtual bool more() const=0;
	virtual bool atFirst() const=0;
	virtual bool atLast() const=0;
	virtual bool atRelative(unsigned long) const=0;
	virtual unsigned long count() const=0;
	virtual unsigned long countAll() const=0;

// sorting public interface
   virtual void setSortOrder(dbSorter* adoptSorter)=0;
   virtual dbSorter* sorter() const=0;
   virtual void sortSelectionNow(fieldNumT)=0;
   virtual void sortSelectionInverseNow(fieldNumT)=0;
   virtual void sortSelectionNow(const dbSorter*)=0;


// data access
	virtual void newRecord()=0;
	virtual void deleteRecord()=0;
	virtual void saveRecord()=0;
	virtual void unloadRecord()=0;
	virtual bool isRecordLoaded() const=0;
	virtual void *getFieldWriteDest(fieldNumT)=0;
	virtual void *getFieldReadFrom(fieldNumT)=0;
	virtual void setBlobLength(fieldNumT, unsigned long len)=0;
	virtual void loadBlob(const dbBLOB*)=0;
	bool isDirty() const;
   void markDirty();

// reflective operators
	virtual unsigned int numIndexes() const=0;
	virtual unsigned int numFiles() const=0;
	virtual unsigned long fieldDataLen(const dbField*) const=0;
	virtual unsigned long blobPointerReferenceSize() const=0;
	virtual long sequenceNumber() const=0;

// public search functions
    virtual unsigned long recordNumber() const=0;
	virtual bool gotoRecord(unsigned long relativeNum)=0;
	virtual void selectAll()=0;
	virtual void selectNone()=0;
	virtual bool search(const dbQueryClause*)=0;
	virtual bool searchSelection(const dbQueryClause*)=0;
	virtual bool searchEqual(const dbField*, const char*, bool matchEntireKey=true)=0;
	virtual bool searchEqual(const dbField*, const void*)=0;
	virtual void difference_with(const OOF_tableBackend*)=0;
	virtual void intersection_with(const OOF_tableBackend*)=0;  
	virtual void invert()=0;  
	virtual void union_with(const OOF_tableBackend*)=0;  
	virtual bool searchSelContainsAnyDelimited(const dbField* schField, const char *schStr, char delim)=0;
	virtual bool searchSelContainsAnyOf(const dbField* schField, const char **schStrs, unsigned long count)=0;
	virtual bool searchSelContainsAllDelimited(const dbField* schField, const char *schStr, char delim)=0;
	virtual bool searchSelContainsAllOf(const dbField* schField, const char **schStrs, unsigned long count)=0;
	virtual bool loadRelatedContextJoiningFromTo(const dbField*, const dbField*)=0;

 // selection of specific contexts
	virtual void selectJustRecord(unsigned long recNo)=0;
	virtual void selectJustOID(oidT)=0;
	virtual void selectJustCurrent()=0;

 protected:
	virtual void CachingContextChange()=0;
	
 // data storage
	dbField *mOIDfield;
	OOF_Dictionary	 mFields;		// copy of dbTable handle object, one representation
	dbTable* mTable;
	bool mDirty;

	friend class dbTable;
};


class OOF_Segment;
class OOF_FieldSegOwner {
public:
// constructors
	OOF_FieldSegOwner();
	OOF_FieldSegOwner(const OOF_FieldSegOwner&);
	virtual ~OOF_FieldSegOwner() {};

	
	virtual unsigned short countSegments() const;
	virtual void addSegment(dbField&);
	OOF_FieldSegOwner& operator<<(dbField&);
	OOF_FieldSegOwner& operator>>(dbField&);

// reflective operators
	OOF_Segment* segment(unsigned int) const;
	fieldNumT fieldNumber(unsigned int) const;

// data storage
private:
	OOF_Dictionary	mSegments;
}; 


class dbSorter : public dbSingleListener, public OOF_FieldSegOwner
{
public:
	dbSorter(dbTable* listensTo=0, OOF_tableBackend* sorts=0);
	dbSorter(const dbSorter&, dbTable* listensTo=0, OOF_tableBackend* sorts=0);
	virtual ~dbSorter() {};

	dbSorter& operator<<(dbField&);
	dbSorter& operator>>(dbField&);
	
	virtual void receiveMsg(OOFmsgT msg, unsigned long senderDefined);
	virtual unsigned long indexNo() const;
	void setSortOnSaves(bool willSortOnSaves=true);

	void suspendSorting();
	void resumeSorting();

	const dbField* field(unsigned int) const;
	
// must implement the following in concrete classes	
	virtual dbSorter* clone(dbTable*, OOF_tableBackend*) const;
	virtual void sortNow();
	virtual unsigned short sortFieldNo() const;
	virtual bool sortsBySingleField() const; 
	
protected:
// data storage
	dbTable* mTable;
	OOF_tableBackend* mSorts;
	bool mSortOnSaves, mNeedsToSort;
};


class reverse
{
public:
	reverse(dbField& fld) : mField(fld) {};
	reverse(dbField* fld) : mField(*fld) {};
// use default copy ctor

private:
	dbField& mField;

friend dbSorter& operator<<(dbSorter&, const reverse&);
friend OOF_FieldSegOwner& operator<<(OOF_FieldSegOwner&, const reverse&);
};


class dbFieldSorter : public dbSorter
{
public:
	dbFieldSorter(dbTable* listensTo , OOF_tableBackend*, fieldNumT);
private:
	dbFieldSorter(const dbFieldSorter&) { assert(0); };
public:
	virtual ~dbFieldSorter() {};
	
	virtual dbSorter* clone(dbTable*, OOF_tableBackend*) const;
	virtual void sortNow();
	virtual unsigned short sortFieldNo() const;
	void setReverseSort(bool shouldReverse=true);
	bool sortsInReverse() const;
	virtual bool sortsBySingleField() const;
	void changeSortFieldTo(fieldNumT);
	
protected:
// data storage
	fieldNumT mSortFieldNo;
	bool mReverseSort;
};

class dbCloneCleanup : public dbSingleListener {
public:
	dbCloneCleanup(dbTable* listensTo);
// use default copy ctor
	virtual ~dbCloneCleanup() {};
	
	virtual void receiveMsg(OOFmsgT msg, unsigned long senderDefined);

// data storage
private:
	dbTable* mTable;
};

// include inline definitions
#include "oof1.inl"
#endif


 