// COPYRIGHT 1994 A.D. Software, All rights reserved

// public layer of OOFILE database

// inline definitions


// -------------------------------------------------------
//                      d b C o n n e c t
// -------------------------------------------------------
inline bool 
dbConnect::underConstruction()
{
	return sCurrentlyConstructing!=0;
}



// -------------------------------------------------------
//                      d b T a b l e
// -------------------------------------------------------
#if 0
inline dbTable::operator dbTable*() 
{ 
	return this; 
};
#endif

inline bool 
dbTable::isRecordLoaded() const
{
	return mBackend->isRecordLoaded();
}


inline long 
dbTable::sequenceNumber() const
{
	return mBackend->sequenceNumber();
}


inline unsigned int 
dbTable::numIndexes() const
{
	return mBackend->numIndexes();
}


inline unsigned int 
dbTable::numFiles() const
{
	return mBackend->numFiles();
}


inline void 
dbTable::createTableInConnection(const dbConnect* theDB) 
{
	mBackend->createTableInConnection(theDB);
}


inline void 
dbTable::openTableInConnection(const dbConnect* theDB) 
{
	mBackend->openTableInConnection(theDB);
}


inline bool 
dbTable::more() const
{
	return mBackend->more();
}


inline bool 
dbTable::atFirst() const
{
	return mBackend->atFirst();
}


inline bool 
dbTable::atLast() const
{
	return mBackend->atLast();
}


inline bool 
dbTable::atRelative(unsigned long rel) const
{
	return mBackend->atRelative(rel);
}


inline oidT
dbTable::currentOID()
{
	return mBackend->currentOID();
}


inline void 
dbTable::next() 
{
	ContextChange();
	mBackend->next();
}


inline void 
dbTable::first()
{
	ContextChange();
	mBackend->gotoRecord(0);
}


inline void 
dbTable::prev()
{
	ContextChange();
	mBackend->prev();
}


inline void 
dbTable::last()
{
	ContextChange();
	mBackend->gotoRecord(mBackend->count()-1);
}


inline unsigned long 
dbTable::count()
{
	ValidatePossiblyRelatedContext();
	return mBackend->count();
}


inline unsigned long 
dbTable::countAll()
{
	ValidatePossiblyRelatedContext();
	return mBackend->countAll();
}


inline unsigned long 
dbTable::recordNumber() const
{
	if (isRecordLoaded())		
		return mBackend->recordNumber();
	else
		return 0;
}


inline void 
dbTable::difference_with(const dbTable& rhs)
{
	mBackend->difference_with(rhs.mBackend);
}
	

inline void 
dbTable::operator-=(const dbTable& rhs)
{
	mBackend->difference_with(rhs.mBackend);
}
	

inline void 
dbTable::operator%=(const dbTable& rhs)
{
	mBackend->difference_with(rhs.mBackend);
}


inline void 
dbTable:: intersection_with(const dbTable& rhs)  
{
	mBackend->intersection_with(rhs.mBackend);
}


inline void 
dbTable:: operator&=(const dbTable& rhs)  
{
	mBackend->intersection_with(rhs.mBackend);
}


inline void 
dbTable::invert()
{
	mBackend->invert();
} 


inline void 
dbTable::operator!()
{
	mBackend->invert();
} 


inline void 
dbTable::operator~()
{
	mBackend->invert();
} 


inline void 
dbTable::union_with(const dbTable& rhs)
{
	mBackend->union_with(rhs.mBackend);
}


inline void 
dbTable::operator+=(const dbTable& rhs)
{
	mBackend->union_with(rhs.mBackend);
}


inline void 
dbTable::operator|=(const dbTable& rhs)
{
	mBackend->union_with(rhs.mBackend);
}



inline void 
dbTable::selectAll()
{
	mBackend->selectAll();
	broadcast(OOFmsg_ChangeSelection);
}


inline void 
dbTable::selectNone()
{
	mBackend->selectNone();
	broadcast(OOFmsg_ChangeSelection);
}


inline bool 
dbTable::isDirty() const
{
	return mBackend->isDirty();
}


inline dbField*
dbTable::field(fieldNumT fieldNum) const
{ 
	return (dbField *) mFields.value(fieldNum); 
}


inline dbField*
dbTable::field(const char *fieldName)
{ 
	return  (dbField *) mFields[fieldName]; 
}


inline const OOF_String&
dbTable::tableName() const
{ 
	return  mTableName; 
}


inline void 
dbTable::setName(const char* name)
{ 
	mTableName = name; 
}


inline void 
dbTable::setOIDfield(const dbField &oidfield) const
{ 
	mBackend->setOIDfield(oidfield);
}
 
 
inline void 
dbTable::setSaveOption(const saveOptions opt)
{
	mSaveOption = opt;
}


inline void 
dbTable::setSortOrder(const dbSorter& srt)
{
	mBackend->setSortOrder(new dbSorter(srt, this, mBackend));
}
 
 
inline dbSorter* 
dbTable::sorter() const
{
	return mBackend->sorter();
}
 
 
 inline void 
dbTable::selectJustRecord(unsigned long recNo)
{
	mBackend->selectJustRecord(recNo);
	broadcast(OOFmsg_ChangeSelection);
}


inline void 
dbTable::selectJustOID(oidT oid)
{
	mBackend->selectJustOID(oid);
	broadcast(OOFmsg_ChangeSelection);
}


inline void 
dbTable::selectJustCurrent()
{
	mBackend->selectJustCurrent();
	broadcast(OOFmsg_ChangeSelection);
}


// -------------------------------------------------------
//           O O F _ t a b l e  B a c k e n d 
// -------------------------------------------------------
inline void 
OOF_tableBackend::markDirty()
{
		mDirty=true;
}



// -------------------------------------------------------
//              O O F _ S t r i n g
// -------------------------------------------------------
inline ostream& 
operator<<(ostream& os, const OOF_String& str)
{
	os << (char *) str;
	return os;
}


inline bool 
OOF_String::isEmpty() const
{
	return mBody==0 || mLen==0;
}


// -------------------------------------------------------
//                      d b S o r t e r 
// -------------------------------------------------------
inline void 
dbSorter::setSortOnSaves(bool onSaves)
{
   mSortOnSaves = onSaves;
}


inline dbSorter& 
dbSorter::operator<<(dbField& segField)
{
	addSegment(segField);
	return *this;
}




// -------------------------------------------------------
//                      d b F i e l d S o r t e r 
// -------------------------------------------------------
inline void 
dbFieldSorter::setReverseSort(bool rev)
{
   mReverseSort = rev;
}


inline bool 
dbFieldSorter::sortsInReverse() const
{
   return mReverseSort;
}


// -------------------------------------------------------
//           O O F _ F i e l d S e g O w n e r
// -------------------------------------------------------
inline unsigned short 
OOF_FieldSegOwner::countSegments() const
{	
	return mSegments.count();
}


inline OOF_FieldSegOwner& 
OOF_FieldSegOwner::operator<<(dbField& segField)
{
	addSegment(segField);
	return *this;
}


inline OOF_Segment* 
OOF_FieldSegOwner::segment(unsigned int i) const
{
	assert(i<mSegments.count());
	return (OOF_Segment*)(mSegments.value(i));  // safe downcast
}


 