#ifndef H_OOF3
#define H_OOF3

// COPYRIGHT 1994 A.D. Software, All rights reserved

// OOFILE database non-numeric field-related classes
// see also oof4.hpp

#include "oof1.hpp"
#ifdef _Macintosh
#include <Types.h>
#endif

enum OOF_fieldTypes {charField, textField, dateField, shortField, uShortField, longField, uLongField, realField, blobField, compoundField, boolField, fixedBinaryField, relationshipField};
ostream& operator<<(ostream&, OOF_fieldTypes);

class dbField : public dbClass {
public:
	enum FieldIsStandaloneT {eFieldIsStandalone};  // used as flag to constructors

public:
 	dbField(); 
	dbField(const OOF_IndexOptions); 
	dbField(const char *);
	dbField(const char *, const OOF_IndexOptions);
 	dbField(const dbField&); 
 	dbField(dbField::FieldIsStandaloneT); 
	virtual ~dbField();
	virtual void deleteStandaloneStorage() {};

	virtual void postCloneCleanup() {};
	
//	operator dbField*(); 

// additional setting operators
	void index(const OOF_IndexOptions=kIndexed);
	void setName(const char* name);

// reflective operators
	const char* tableName() const;
	bool isStandalone() const;
	virtual void describe(ostream&);
//	virtual unsigned short countValues() const;
	virtual OOF_fieldTypes fieldType() const=0;
	virtual OOF_fieldTypes nativeType() const;
	virtual unsigned long fieldLen() const=0;
	virtual unsigned long fieldEditLen() const;
	dbTable* fieldTable() const;
	unsigned long fieldDataLen() const;
	fieldNumT fieldNumber() const;
	virtual const OOF_String& fieldName() const;
	OOF_IndexOptions fieldIndexOptions() const;
	bool fieldIndexAllowsDups() const;
	bool fieldIndexIsCompressLeading() const;
	bool fieldIndexIsCompressPadding() const;
	virtual bool fieldIsIndexed() const;
	virtual bool fieldIsUniqueIndexed() const;
	unsigned short fieldNumIndexes() const;
	bool allowsDuplicates() const;
	bool caseSensitive() const;
	virtual bool fieldIsBlob() const;
	virtual bool fieldIsBinary() const;
	virtual bool fieldIsVirtual() const;

// data access
	virtual void setString(const char*) { assert(0); };  
	void operator=(const char*); 
	virtual char *copyAsChars() const;
	virtual const char* asChars() const;
	const void* binaryContents() const;
	const void* currentbinaryContents() const;
	virtual void extract(ostream&) const;
	virtual bool insert(istream&, char fieldSep, char recSep);
	virtual void copyValueFrom(const dbField*) {assert(0); };  // should override but only if makes sense to join on field type
	virtual bool validateContents() { return true; };

	virtual void generateTestData(const char* /*testBuf*/, unsigned long /*testBufLen*/) {};	
	
	
protected:
// data storage
	OOF_tableBackend *mBackend;
	dbTable*	mTable;
	fieldNumT mFieldNumber;
	
private:
	OOF_IndexOptions mIndexOptions;
	OOF_String mFieldName;

	friend	class dbTable;
};
typedef dbField *dbFieldPtr;
istream& operator>>(istream&, dbField*);
istream& operator>>(istream&, dbField&);


class dbQueryBinary;
class dbQueryTrinary;
class dbQueryBinaryCombo;

class dbChar : public dbField  {
public:
// constructors
	dbChar(long fieldWidth = 80);
	dbChar(long fieldWidth, const char *fieldName);
	dbChar(long fieldWidth, const OOF_IndexOptions indexOptions);
	dbChar(long fieldWidth, const char *fieldName, const OOF_IndexOptions indexOptions);
 	dbChar(dbField::FieldIsStandaloneT k, long fieldWidth = 80); 
	dbChar(const dbChar&);
	virtual ~dbChar();
	virtual void deleteStandaloneStorage();

// reflective operators
	virtual void describe(ostream&);
	virtual OOF_fieldTypes fieldType() const;
	virtual bool fieldIsBinary() const;
	virtual unsigned long fieldLen() const;
	unsigned short length() const;

// search factories
	dbQueryBinary	operator==(const char*) const;
	dbQueryBinary	operator<(const char*) const;
	dbQueryBinary	operator<=(const char*) const;
	dbQueryBinary	operator>=(const char*) const;
	dbQueryBinary	operator>(const char*) const;
	dbQueryBinary	operator!=(const char*) const;
	dbQueryBinary	startsWith(const char*) const;
//	dbQueryBinary	contains(const char*) const;
	dbQueryTrinary	between(const char*, const char*) const;
	dbQueryTrinary	outside(const char*, const char*) const;

// data access

// getters
	virtual const char* asChars() const;
	virtual  char * copyAsChars() const;
	operator const char*() const;
	virtual void extract(ostream& os) const;
	virtual void copyValueFrom(const dbField*);
#ifdef _Macintosh
	void asStr255(Str255 s) const;
#endif

// setters
#ifdef _Macintosh
	void setStr255(const Str255 s);
#endif
	virtual void setString(const char*);  
	dbChar& operator=(const char*);  
	void setChars(const char* str , unsigned long charLen);
	dbChar& operator=(const dbChar&);
	virtual bool insert(istream&, char fieldSep, char recSep);
	char& operator[](unsigned short);
	virtual void generateTestData(const char *testBuf, unsigned long testBufLen);

private:
	void CompleteInit();
	
// data storage
 unsigned long mMaxLength;
};


class dbBLOB;
class dbBLOBstorage {
private:  // owned totally by a dbBLOB, only exposed here to allow inline definitions
// constructors
	dbBLOBstorage(dbBLOB* owner);	
	dbBLOBstorage(const dbBLOBstorage&) { assert(0); };
	virtual ~dbBLOBstorage();

// reflective operators
	bool isDirty() const;

// data access
	void *bodyAddress() const;
	void reset();
	void* allocRoomFor(unsigned long);
	void *orphanBody();
	void adoptBody(void* body, unsigned long bodyLen);
	void markDirty();
	void setChars(const char* str , unsigned long charLen);

protected:
// data storage
	void*	mBody;	// owned
	unsigned long mBodyLength;  // allow for part-using the block
	bool mDirty,mLoaded;
	dbBLOB* mOwner;
	
friend class dbBLOB;
friend class dbText;
};


class dbBLOB : public dbField {
public:
// constructors
	dbBLOB();	
	dbBLOB(const OOF_IndexOptions indexOptions);
	dbBLOB(const char *fieldName) ;
	dbBLOB(const char *fieldName, const OOF_IndexOptions indexOptions);
 	dbBLOB(dbField::FieldIsStandaloneT k) : dbField(k) {}; 
	dbBLOB(const dbBLOB&);
	virtual ~dbBLOB();

// reflective operators
	virtual OOF_fieldTypes fieldType() const;
	virtual unsigned long fieldLen() const;
	virtual bool fieldIsBlob() const;
	bool isDirty() const;
	bool loaded() const;

// data access - all const because handled by separate storage
	void *bodyAddress() const;
	void reset() const;
	void* allocRoomFor(unsigned long) const;
	void *orphanBody() const;
	void adoptBody(void* body, unsigned long bodyLen) const;
	void markDirty() const;

protected:
	 void SetLength(unsigned long len);
	void LoadField() const;
	
// data storage
	dbBLOBstorage*	mStorage;	// owned
	
friend void dbBLOBstorage::setChars(const char* str , unsigned long charLen);
};


class dbText : public dbBLOB {
public:
// constructors
	dbText(){};
	dbText(const OOF_IndexOptions);
	dbText(const char *fieldName);
	dbText(const char *fieldName, const OOF_IndexOptions);
 	dbText(dbField::FieldIsStandaloneT k) : dbBLOB(k) {}; 
// use default copy ctor
	virtual ~dbText() {};

// reflective operators
	virtual OOF_fieldTypes fieldType() const;
	virtual OOF_fieldTypes nativeType() const;
	virtual bool fieldIsBinary() const;

// search factories
	dbQueryBinary	operator==(const char*) const;
	dbQueryBinary	operator<(const char*) const;
	dbQueryBinary	operator<=(const char*) const;
	dbQueryBinary	operator>=(const char*) const;
	dbQueryBinary	operator>(const char*) const;
	dbQueryBinary	operator!=(const char*) const;
//	dbQueryBinary	contains(const char*) const;

// data access
	virtual const char* asChars() const;
	virtual char * copyAsChars() const;
	operator const char*() const;
	virtual void setString(const char*);  
	dbText& operator=(const char*); 
	dbText& operator=(const dbText&);
	void operator+=(const char*);  // the only field with an append operator
	void setChars(const char* str , unsigned long charLen);
	virtual void extract(ostream& os) const;
	virtual bool insert(istream&, char fieldSep, char recSep);
	virtual void copyValueFrom(const dbField*);

	virtual void generateTestData(const char *testBuf, unsigned long testBufLen);	
};


class dbFixedBinary : public dbField {
public:
// constructors
	dbFixedBinary(long fieldWidth = 4);
	dbFixedBinary(long fieldWidth, const OOF_IndexOptions indexOptions); 
	dbFixedBinary(long fieldWidth, const char *fieldName);  
	dbFixedBinary(long fieldWidth, const char *fieldName, const OOF_IndexOptions indexOptions);  
	// use default copy ctor
	dbFixedBinary& operator=(const dbFixedBinary& rhs);
 	dbFixedBinary(dbField::FieldIsStandaloneT k) : dbField(k) {}; 
	virtual ~dbFixedBinary() {};

// reflective operators
	virtual OOF_fieldTypes fieldType() const;
	virtual unsigned long fieldLen() const;
	
// data access
	dbFixedBinary&  operator=(const char* str);
	virtual void setString(const char*);  
	void setValue(const void* str);
	void setByte(char c, unsigned long offset);
	virtual void extract(ostream& os) const;
	virtual void generateTestData(const char *testBuf, unsigned long testBufLen);
	virtual void describe(ostream& os);

protected:
	
// data storage
	unsigned long mBodyLength; 
	

};


class dbCompoundField : public dbField, public OOF_FieldSegOwner {
public:
// constructors
	dbCompoundField();
	dbCompoundField(const OOF_IndexOptions);
	dbCompoundField(const char *fieldName);
	dbCompoundField(const char *fieldName, const OOF_IndexOptions);
	dbCompoundField(const dbCompoundField&);
	virtual ~dbCompoundField() {};

	virtual void addSegment(dbField&);
		
// data output
	virtual void extract(ostream& os) const;

// reflective operators
	virtual OOF_fieldTypes fieldType() const;
	virtual unsigned long fieldLen() const;
	virtual bool fieldIsVirtual() const;

	const dbField* field(unsigned int) const;

// search factories
	dbQueryBinary	operator==(const char*) const;
	dbQueryBinary	operator<(const char*) const;
	dbQueryBinary	operator<=(const char*) const;
	dbQueryBinary	operator>=(const char*) const;
	dbQueryBinary	operator>(const char*) const;
	dbQueryBinary	operator!=(const char*) const;
	dbQueryBinary	startsWith(const char* str) const;

// data storage
private:
	unsigned int mTotalLength;
};


class dbFieldCopier : public dbSingleListener {
public:
	dbFieldCopier(dbBroadcaster* listensTo, unsigned short copyOnMessage, dbField* copyFrom, dbField* copyTo);
	// use default copy ctor
	virtual ~dbFieldCopier() {};
	
	virtual void receiveMsg(OOFmsgT msg, unsigned long senderDefined);
protected:
// data storage
	dbField *mCopyFrom, *mCopyTo;
	OOFmsgT mCopyOnMessage;
};


class OOF_Segment : public dbClass 
{
private:  // yep, wholly owned
	OOF_Segment(const dbField& fld);
	OOF_Segment(const dbField* fld);
// use default copy ctor
	~OOF_Segment() {};

public:
	void setReverse();
	bool isReversed() const;
	
private:	
// data storage
	fieldNumT mFieldNo;
	bool mReversed;
	
friend class OOF_FieldSegOwner;
};
// include inline definitions
#include "oof3.inl"
#endif
