// COPYRIGHT 1994 A.D. Software, All rights reserved

// OOFILE database field-related classes
// numeric fields - see also oof3.cpp

#include "oof2.hpp"
#include "oof4.hpp"
#include "oofquery.hpp"  
#include "oofrel.hpp" 

extern "C" {
#include <ctype.h>
#include <time.h>  // for dbDate stuff
}

// define static variables
dbDate::dateFieldOrder dbDate::sDefaultDateOrder=orderDMY;


// -------------------------------------------------------
//               d b N u m e r i c F i e l d  
// -------------------------------------------------------
dbNumericField::dbNumericField(const OOF_IndexOptions indexOptions) :
	dbField(indexOptions)
{}


dbNumericField::dbNumericField(const char *fieldName)  :
	dbField(fieldName)
{}


dbNumericField::dbNumericField(const char *fieldName, const OOF_IndexOptions indexOptions)  :
	dbField(fieldName, indexOptions)
{}


dbQueryTrinary	
dbNumericField::between(long fromNum, long toNum) const
{
	return dbQueryTrinary(new dbQueryField(this), dbQueryClause::between,
		MakeQueryLiteral(fromNum), MakeQueryLiteral(toNum));
}


dbQueryTrinary	
dbNumericField::outside(long fromNum, long toNum) const
{
	return dbQueryTrinary(new dbQueryField(this), dbQueryClause::outside,
		MakeQueryLiteral(fromNum), MakeQueryLiteral(toNum));
}


dbQueryBinary	
dbNumericField::operator==(long n) const
{
	return dbQueryBinary(new dbQueryField(this), dbQueryClause::equals, MakeQueryLiteral(n));
}    


dbQueryBinary	
dbNumericField::operator!=(long n) const
{
	return dbQueryBinary(new dbQueryField(this), dbQueryClause::notEquals, MakeQueryLiteral(n));
}  


dbQueryBinary	
dbNumericField::operator<(long n) const
{
	return dbQueryBinary(new dbQueryField(this), dbQueryClause::lessThan, MakeQueryLiteral(n));
}  


dbQueryBinary	
dbNumericField::operator<=(long n) const
{
	return dbQueryBinary(new dbQueryField(this), dbQueryClause::lessThanOrEqual, MakeQueryLiteral(n));
}  


dbQueryBinary	
dbNumericField::operator>(long n) const
{
	return dbQueryBinary(new dbQueryField(this), dbQueryClause::greaterThan, MakeQueryLiteral(n));
}  


dbQueryBinary	
dbNumericField::operator>=(long n) const
{
	return dbQueryBinary(new dbQueryField(this), dbQueryClause::greaterThanOrEqual, MakeQueryLiteral(n));
}  


void 
dbNumericField::operator/=(long)

{
// NOT YET IMPLEMENTED
}



void 
dbNumericField::operator-=(long)

{
// NOT YET IMPLEMENTED
}



void 
dbNumericField::operator+=(long)

{
// NOT YET IMPLEMENTED
}



void 
dbNumericField::operator*=(long)

{
// NOT YET IMPLEMENTED
;
}


void 
dbNumericField::operator/=(double)

{
// NOT YET IMPLEMENTED
}



void 
dbNumericField::operator-=(double)

{
// NOT YET IMPLEMENTED
}



void 
dbNumericField::operator+=(double)

{
// NOT YET IMPLEMENTED
}



void 
dbNumericField::operator*=(double)

{
// NOT YET IMPLEMENTED
;
}



void 
dbNumericField::CheckRange(double d, long minL, long maxL, const char* file, int line)
{
	if (d<minL) {
		dbConnect::raise(ostrstream() << file << "line: " << line <<
		"Range conversion error: double " << d 
		<< " below min of " << minL << endl);
	}
	else
		if (d>maxL) {
			dbConnect::raise(ostrstream() << file << "line: " << line <<
			"Range conversion error: double " << d 
			<< " above max of " << maxL << endl);
		}
}
 
 

void 
dbNumericField::CheckRange(long l, long min, long max, const char* file, int line)
{
	if (l<min) {
		dbConnect::raise(ostrstream() << file << "line: " << line <<
		"Range conversion error: long " << l 
		<< " below min of " << min << endl);
	}
	else
		if (l>max) {
			dbConnect::raise(ostrstream() << file << "line: " << line <<
			"Range conversion error: long " << l 
			<< " above max of " << max << endl);
		}
}
 
 
void 
dbNumericField::CheckRange(unsigned long l, unsigned long max, const char* file, int line)
{
	if (l>max) {
		dbConnect::raise(ostrstream() << file << "line: " << line <<
		"Range conversion error: ulong " << l 
		<< " above max of " << max << endl);
	}
}
 
 
dbQueryLiteral* 
dbNumericField::MakeQueryLiteral(long n) const
{
	return new dbQueryLiteralLong(n);
}


// -------------------------------------------------------
//                   d b S h o r t
// -------------------------------------------------------
dbShort::dbShort(const OOF_IndexOptions indexOptions) :
	dbNumericField(indexOptions)
{}


dbShort::dbShort(const char *fieldName)  :
	dbNumericField(fieldName)
{}


dbShort::dbShort(const char *fieldName, const OOF_IndexOptions indexOptions)  :
	dbNumericField(fieldName, indexOptions)
{}


dbShort& 
dbShort::operator=(const dbShort& rhs)
{
	if (this == &rhs)
		return *this;
	
	*this = (short)rhs;
	return *this;
}


void 
dbShort::setString(const char* str)
{
	short n;
	istrstream is((char*)str);  // cast away const for Borland
	is >> n;
	*this = n;
}


dbShort::operator short() const
{
	if (isStandalone()) {
		short ret = (long) mBackend;
		return ret;
	}
	else {
		mTable->validateContext();
		short *readFrom = (short *)mBackend->getFieldReadFrom(mFieldNumber);
		return *readFrom;
	}
}


dbShort& 
dbShort::operator=(short n)
{
	if (isStandalone())
		mBackend = (OOF_tableBackend*) n;
	else {
		mTable->validateContext();
		short *writeTo = (short *)mBackend->getFieldWriteDest(mFieldNumber);
		if (writeTo)
			*writeTo = n;	
		mBackend->markDirty();
	}
	return *this;
}


void 
dbShort::copyValueFrom(const dbField* srcField)
{
	if (srcField->fieldType()==shortField)
		*this = *(dbShort*)srcField;  // safe downcast
#ifdef OOF_Debug
	else
		dbConnect::raise(ostrstream() << "wrong field type passed in to dbShort::copyValueFrom, copying to " << fieldName() << " from " << srcField->fieldName());
#endif
}


OOF_fieldTypes 
dbShort::fieldType() const
{
	return shortField;
}


unsigned long 
dbShort::fieldLen() const
{
	return sizeof(short); 
}


void 
dbShort::extract(ostream& os) const
{
	os << (short) *this;
}


bool 
dbShort::insert(istream& is, char fieldSep, char recSep)
{
	// is the stream good?
	if (!is.good())
		return false;
	// read in the short
	short n;
	is >> n;
	// read in a potential field/record separator
	char c;
	is >> c;
	bool hitEndOfField = false;
	if ((c==fieldSep) || (c==recSep)) 
		hitEndOfField = true;
	// put back the potential field/record separator
	is.putback(c);
	if (hitEndOfField)
		*this = n;
	return hitEndOfField;
}


void 
dbShort::generateTestData(const char* /*testBuf*/, unsigned long /*testBufLen*/)
{
	*this = (short)(rand()-rand()/2);
}


dbQueryLiteral* 
dbShort::MakeQueryLiteral(long n) const
{
	return new dbQueryLiteralShort((short)n);
}


// -------------------------------------------------------
//                   d b U s h o r t
// -------------------------------------------------------
dbUshort::dbUshort(const OOF_IndexOptions indexOptions) :
	dbNumericField(indexOptions)
{}

 
dbUshort::dbUshort(const char *fieldName)  :
	dbNumericField(fieldName)
{}

 
dbUshort::dbUshort(const char *fieldName, const OOF_IndexOptions indexOptions)  :
	dbNumericField(fieldName, indexOptions)
{}

 
dbUshort& 
dbUshort::operator=(const dbUshort& rhs)
{
	if (this == &rhs)
		return *this;
	
	*this = (unsigned short)rhs;
	return *this;
}


void 
dbUshort::setString(const char* str)
{
	unsigned short n;
	istrstream is((char*)str);  // cast away const for Borland
	is >> n;
	*this = n;
}


dbUshort::operator unsigned short() const
{
	if (isStandalone()) {
		unsigned short ret = (unsigned long) mBackend;
		return ret;
	}
	else {
		mTable->validateContext();
		unsigned short *readFrom = (unsigned short *)mBackend->getFieldReadFrom(mFieldNumber);
		return *readFrom;
	}
}


dbUshort& 
dbUshort::operator=(unsigned short n)
{
	if (isStandalone())
		mBackend = (OOF_tableBackend*) n;
	else {
		mTable->validateContext();
		unsigned short *writeTo = (unsigned short *)mBackend->getFieldWriteDest(mFieldNumber);
		if (writeTo)
			*writeTo = n;	
		mBackend->markDirty();
	}
	return *this;
}


void 
dbUshort::copyValueFrom(const dbField* srcField)
{
	if (srcField->fieldType()==uShortField)
		*this = *(dbUshort*)srcField;  // safe downcast
#ifdef OOF_Debug
	else
		dbConnect::raise(ostrstream() << "wrong field type passed in to dbUshort::copyValueFrom, copying to " << fieldName() << " from " << srcField->fieldName());
#endif
}


OOF_fieldTypes 
dbUshort::fieldType() const
{
	return uShortField;
}


unsigned long 
dbUshort::fieldLen() const
{
	return sizeof(unsigned short); 
}


void 
dbUshort::extract(ostream& os) const
{
	os << (unsigned short)*this;
}


bool 
dbUshort::insert(istream& is, char fieldSep, char recSep)
{
	// is the stream good?
	if (!is.good())
		return false;
	// read in the unsigned short
	unsigned short n;
	is >> n;
	// read in a potential field/record separator
	char c;
	is >> c;
	bool hitEndOfField = false;
	if ((c==fieldSep) || (c==recSep)) 
		hitEndOfField = true;
	// put back the potential field/record separator
	is.putback(c);
	if (hitEndOfField)
		*this = n;
	return hitEndOfField;
}



void 
dbUshort::generateTestData(const char* /*testBuf*/, unsigned long /*testBufLen*/)
{
	*this = (unsigned short)(rand());
}


dbQueryLiteral* 
dbUshort::MakeQueryLiteral(long n) const
{
	return new dbQueryLiteralUshort((unsigned short)n);
}


// -------------------------------------------------------
//                      d b L o n g
// -------------------------------------------------------
dbLong::dbLong(const OOF_IndexOptions indexOptions) :
	dbNumericField(indexOptions)
{}


dbLong::dbLong(const char *fieldName)  :
	dbNumericField(fieldName)
{}


dbLong::dbLong(const char *fieldName, const OOF_IndexOptions indexOptions)  :
	dbNumericField(fieldName, indexOptions)
{}


dbLong& 
dbLong::operator=(const dbLong& rhs)
{
	if (this == &rhs)
		return *this;
	
	*this = (long)rhs;
	return *this;
}


dbLong& 
dbLong::operator=(long n)
{
	if (isStandalone())
		mBackend = (OOF_tableBackend*) n;
	else {
		mTable->validateContext();
		long *writeTo = (long *)mBackend->getFieldWriteDest(mFieldNumber);
		if (writeTo)
			*writeTo = n;	
		mBackend->markDirty();
	}
	return *this;
}


void 
dbLong::setString(const char* str)
{
	long n;
	istrstream is((char*)str);  // cast away const for Borland
	is >> n;
	*this = n;
}


dbLong::operator long() const
{
	if (isStandalone()) 
		return (long) mBackend;
	else {
		mTable->validateContext();
		long *readFrom = (long *)mBackend->getFieldReadFrom(mFieldNumber);
		return *readFrom;
	}
}


void 
dbLong::copyValueFrom(const dbField* srcField)
{
	if (srcField->fieldType()==longField)
		*this = *(dbLong*)srcField;  // safe downcast
#ifdef OOF_Debug
	else
		dbConnect::raise(ostrstream() << "wrong field type passed in to dbLong::copyValueFrom, copying to " << fieldName() << " from " << srcField->fieldName());
#endif
}


OOF_fieldTypes 
dbLong::fieldType() const
{
	return longField; 
}


unsigned long 
dbLong::fieldLen() const
{
	return sizeof(long); 
}


void 
dbLong::extract(ostream& os) const
{
	os << (long) *this;
}


bool 
dbLong::insert(istream& is, char fieldSep, char recSep)
{
	// is the stream good?
	if (!is.good())
		return false;
	// read in the long
	long n;
	is >> n;
	// read in a potential field/record separator
	char c;
	is >> c;
	bool hitEndOfField = false;
	if ((c==fieldSep) || (c==recSep)) 
		hitEndOfField = true;
	// put back the potential field/record separator
	is.putback(c);
	if (hitEndOfField)
		*this = n;
	return hitEndOfField;
}


void 
dbLong::generateTestData(const char* /*testBuf*/, unsigned long /*testBufLen*/)
{
	*this = (rand()-rand());
}


// -------------------------------------------------------
//                      d b U l o n g
// -------------------------------------------------------
dbUlong::dbUlong(const OOF_IndexOptions indexOptions) :
	dbNumericField(indexOptions)
{}


dbUlong::dbUlong(const char *fieldName)  :
	dbNumericField(fieldName)
{}


dbUlong::dbUlong(const char *fieldName, const OOF_IndexOptions indexOptions)  :
	dbNumericField(fieldName, indexOptions)
{}


dbUlong& 
dbUlong::operator=(const dbUlong& rhs)
{
	if (this == &rhs)
		return *this;
	
	*this = (unsigned long)rhs;
	return *this;
}


dbUlong& 
dbUlong::operator=(unsigned long n)
{
	if (isStandalone())
		mBackend = (OOF_tableBackend*) n;
	else {
		mTable->validateContext();
		unsigned long *writeTo = (unsigned long *)mBackend->getFieldWriteDest(mFieldNumber);
		if (writeTo)
			*writeTo = n;	
		mBackend->markDirty();
	}
	return *this;
}


void 
dbUlong::setString(const char* str)
{
	unsigned long n;
	istrstream is((char*)str);  // cast away const for Borland
	is >> n;
	*this = n;
}


dbUlong::operator unsigned long() const
{
	if (isStandalone()) 
		return (unsigned long) mBackend;
	else {
		mTable->validateContext();
		unsigned long *readFrom = (unsigned long *)mBackend->getFieldReadFrom(mFieldNumber);
		return *readFrom;
	}
}


void 
dbUlong::copyValueFrom(const dbField* srcField)
{
	if (srcField->fieldType()==uLongField)
		*this = *(dbUlong*)srcField;  // safe downcast
#ifdef OOF_Debug
	else
		dbConnect::raise(ostrstream() << "wrong field type passed in to dbUlong::copyValueFrom, copying to " << fieldName() << " from " << srcField->fieldName());
#endif
}


OOF_fieldTypes 
dbUlong::fieldType() const
{
	return uLongField; 
}


unsigned long 
dbUlong::fieldLen() const
{
	return sizeof(unsigned long); 
}


void 
dbUlong::extract(ostream& os) const
{
	os << (unsigned long)*this;
}


bool 
dbUlong::insert(istream& is, char fieldSep, char recSep)
{
	// is the stream good?
	if (!is.good())
		return false;
	// read in the unsigned long
	unsigned long n;
	is >> n;
	// read in a potential field/record separator
	char c;
	is >> c;
	bool hitEndOfField = false;
	if ((c==fieldSep) || (c==recSep)) 
		hitEndOfField = true;
	// put back the potential field/record separator
	is.putback(c);
	if (hitEndOfField)
		*this = n;
	return hitEndOfField;
}


void 
dbUlong::generateTestData(const char* /*testBuf*/, unsigned long /*testBufLen*/)
{
	*this = (unsigned long)rand();
}


dbQueryLiteral* 
dbUlong::MakeQueryLiteral(unsigned long n) const
{
	return new dbQueryLiteralUlong(n);
}


dbQueryLiteral* 
dbUlong::MakeQueryLiteral(long n) const
{
	return new dbQueryLiteralUlong((unsigned long) n);
}


dbQueryBinary	
dbUlong::operator==(unsigned long n) const
{
	return dbQueryBinary(new dbQueryField(this), dbQueryClause::equals, MakeQueryLiteral(n));
}


// -------------------------------------------------------
//                      d b D a t e
// -------------------------------------------------------
// prototype private func
void skipStreamPattern(istream&, const char *);

dbDate::dbDate(const OOF_IndexOptions indexOptions) :
	dbNumericField(indexOptions)
{}


dbDate::dbDate(const char *fieldName)  :
	dbNumericField(fieldName)
{}


dbDate::dbDate(const char *fieldName, const OOF_IndexOptions indexOptions)  :
				dbNumericField(fieldName, indexOptions)
{}


dbDate::dbDate(const dbDate& rhs) : 
	dbNumericField(rhs)
{}


dbDate::~dbDate()
{
}

 
unsigned long 
dbDate::fieldLen() const
{
	return sizeof(unsigned long); 
}


void 
dbDate::copyValueFrom(const dbField* srcField)
{
	if (srcField->fieldType()==dateField)
		*this = *(dbDate*)srcField;  // safe downcast
#ifdef OOF_Debug
	else
		dbConnect::raise(ostrstream() << "wrong field type passed in to dbDate::copyValueFrom, copying to " << fieldName() << " from " << srcField->fieldName());
#endif
}


OOF_fieldTypes 
dbDate::fieldType() const
{
	return dateField; 
}


OOF_fieldTypes 
dbDate::nativeType() const
{
	return uLongField; 
}


void 
dbDate::extract(ostream& os) const
{
	unsigned long binDate = (long)*this;
	unsigned short day, month, year;
	long2ymd(binDate, year, month, day);
	ymd2Stream(year, month, day, os);
}


bool 
dbDate::insert(istream& is, char fieldSep, char recSep)
{
	// is the stream good?
	if (!is.good())
		return false;
	// read in the date
	unsigned short y,m,d;
	istream2ymd(is,y,m,d,fieldSep,recSep,sDefaultDateOrder);
	// read in a potential field/record separator
	char c;
	is >> c;
	bool hitEndOfField = false;
	if ((c==fieldSep) || (c==recSep)) 
		hitEndOfField = true;
	// put back the potential field/record separator
	is.putback(c);
	if (hitEndOfField)
		setDate(y,m,d);
	return hitEndOfField;
}


ostream& 
dbDate::today(ostream& os)
{
	unsigned short day, month, year;
	currentDate2ymd(year, month, day);
	return ymd2Stream(year, month, day, os);
}


ostream& 
dbDate::ymd2Stream(unsigned short year, unsigned short month, unsigned short day, ostream& os)
{
	switch (sDefaultDateOrder) {
	case orderYMD :
		os << year << "/" << month << "/" << day;
		break;
		
	case orderDMY : 
		os << day << "/" << month << "/" << year;
		break;
		
	case orderMDY : 
		os << month << "/" << day << "/" << year;
		break;
	}
	return os;
}


void 
dbDate::generateTestData(const char* /*testBuf*/, unsigned long /*testBufLen*/)
{
//	const long startDate = ymd2Long(1963, 5, 7);
//	const long endDate = ymd2Long(2001,1,1);
//	const long dateDiff = endDate - startDate;
//	setDate(startDate);
//  operator+=(rand()%dateDiff);   // don't have this operator yet!

// pretty sick alternative but at least it generates legal dates
	setDate(ymd2Long( (rand()%(2010-1963) + 1963),
	                  (rand()%12), (rand()%28) ));
}


dbQueryTrinary	
dbDate::between(const char* fromStr, const char* toStr) const
{
	return dbNumericField::between(chars2Long(fromStr), chars2Long(toStr));
}


dbQueryTrinary	
dbDate::outside(const char* fromStr, const char* toStr) const
{
	return dbNumericField::outside(chars2Long(fromStr), chars2Long(toStr));
}


dbQueryBinary	
dbDate::operator==(const char* str) const
{
	return dbNumericField::operator==(chars2Long(str));
}    


dbQueryBinary	
dbDate::operator!=(const char* str) const
{
	return dbNumericField::operator!=(chars2Long(str));
}  


dbQueryBinary	
dbDate::operator<(const char* str) const
{
	return dbNumericField::operator<(chars2Long(str));
}  


dbQueryBinary	
dbDate::operator<=(const char* str) const
{
	return dbNumericField::operator<=(chars2Long(str));
}  


dbQueryBinary	
dbDate::operator>(const char* str) const
{
	return dbNumericField::operator>(chars2Long(str));
}  


dbQueryBinary	
dbDate::operator>=(const char* str) const
{
	return dbNumericField::operator>=(chars2Long(str));
}  


dbQueryTrinary	
dbDate::between(const dbDate& fromDate, const dbDate& toDate) const
{
	return dbNumericField::between((long)fromDate, (long)toDate);
}


dbQueryTrinary	
dbDate::outside(const dbDate& fromDate, const dbDate& toDate) const
{
	return dbNumericField::outside((long)fromDate, (long)toDate);
}


dbQueryBinary	
dbDate::operator==(const dbDate& rhs) const
{
	return dbNumericField::operator==((long)rhs);
}    


dbQueryBinary	
dbDate::operator!=(const dbDate& rhs) const
{
	return dbNumericField::operator!=((long)rhs);
}  


dbQueryBinary	
dbDate::operator<(const dbDate& rhs) const
{
	return dbNumericField::operator<((long)rhs);
}  


dbQueryBinary	
dbDate::operator<=(const dbDate& rhs) const
{
	return dbNumericField::operator<=((long)rhs);
}  


dbQueryBinary	
dbDate::operator>(const dbDate& rhs) const
{
	return dbNumericField::operator>((long)rhs);
}  


dbQueryBinary	
dbDate::operator>=(const dbDate& rhs) const
{
	return dbNumericField::operator>=((long)rhs);
}  


dbDate& 
dbDate::operator=(const char* str)
{
	unsigned short day, month, year;
	chars2ymd(str, year, month, day);
	setDate(year, month, day);
	return *this;
}


dbDate& 
dbDate::operator=(const dbDate& rhs)
{
	if (this == &rhs)
		return *this;
	
	setDate((long)rhs);
	return *this;
}


void 
dbDate::setDate(unsigned long n)
{
	if (isStandalone()) 
		mBackend = (OOF_tableBackend*) n;
	else {
		mTable->validateContext();
		unsigned long *writeTo = (unsigned long *)mBackend->getFieldWriteDest(mFieldNumber);
		if (writeTo)
			*writeTo = n;	
		mBackend->markDirty();
	}
}


#ifdef _Macintosh
void 
dbDate::asStr255(Str255 s) const
{
	char* temp = copyAsChars();
	s[0] = strlen(temp);
	memcpy(s+1, temp, s[0]);
	delete[] temp;
#ifdef OOF_SmartHeap
	assert(MemPoolCheck(MemDefaultPool));
#endif
}


void 
dbDate::setStr255(const Str255 s)
{
	const char* strStart = (char*) s+1;
	dbDate::operator=(strStart);
}



// end of Mac-specific extra conversions
#endif


dbDate::operator unsigned long() const
{
	if (isStandalone()) 
		return (unsigned long) mBackend;
	else {
		mTable->validateContext();
		unsigned long *readFrom = (unsigned long *)mBackend->getFieldReadFrom(mFieldNumber);
		return *readFrom;
	}
}


void 
dbDate::setDate(unsigned short year, unsigned short month, unsigned short day)
{
	unsigned long binDate = ymd2Long(year, month, day);
	setDate(binDate);
}


void 
dbDate::setDateToCurrentDate()
{
	unsigned short year, month, day;
	currentDate2ymd(year, month, day);
	setDate(year, month, day);
}


void 
dbDate::getDate(unsigned short& year, unsigned short& month, unsigned short& day) const
{
	unsigned long binDate = (long) *this;
	long2ymd(binDate, year, month, day);
}


unsigned long 
dbDate::ymd2Long(unsigned short year, unsigned short month, unsigned short day)
{
	if (checkDate(year, month, day)) {
	if (year<100)
		year += currentCentury();
	return (( ( (( unsigned long)year << 8) + (unsigned long)month) << 8) + day);	// use discrete bytes for ease of reading
	}
	else {
#ifdef OOF_Debug
		dbConnect::raise(ostrstream() << "Bad date passed to dbDate::ymd2Long: YMD = " 
		<< year << '\t' << month << '\t' << day << endl);
#endif	
		return 0;
	}
}


void 
dbDate::long2ymd(unsigned long binaryDate, unsigned short& year, unsigned short& month, unsigned short& day)
{
	day = binaryDate & 0xFF;
	unsigned long shifted = binaryDate >> 8;
	month = shifted & 0xFF;
	shifted = shifted >> 8;
	year = shifted;
}


dbDate 
dbDate::operator-(const dbDate& rhs) const
{
	dbDate ret = *this;
	ret -= rhs;
	return ret;
}



dbDate 
dbDate::operator-(long rhs) const
{
	dbDate ret = *this;
	ret -= rhs;
	return ret;
}


dbDate 
dbDate::operator+(const dbDate& rhs) const
{
	dbDate ret = *this;
	ret += rhs;
	return ret;
}


dbDate 
dbDate::operator+(long rhs) const
{
	dbDate ret = *this;
	ret += rhs;
	return ret;
}


dbDate
dbDate::currentDate()
{
	dbDate ret = dbDate(dbField::eFieldIsStandalone);  // g++ 2.7 disliked just ret(blah)
	ret.setDateToCurrentDate();
	return ret;
}


void 
dbDate::currentDate2ymd(unsigned short& year, unsigned short& month, unsigned short& day)
{
	time_t numTime = time(0);
	if (numTime == -1) {
#ifdef OOF_Debug
		dbConnect::raise("System time not available");
#endif
		year = month = day = 0;
	}
	else {
		struct tm* timeBits;
		timeBits = localtime(&numTime);
		day = timeBits->tm_mday;
		month = timeBits->tm_mon+1;
		year = timeBits->tm_year+1900;
	}
}


unsigned short 
dbDate::currentCentury()
{
	time_t numTime = time(0);
	if (numTime == -1) {
#ifdef OOF_Debug
		dbConnect::raise("System time not available");
#endif
		return 1900;  // coarse default
	}
	else {
		struct tm* timeBits;
		timeBits = localtime(&numTime);
		return (timeBits->tm_year/100)*100+1900;
	}
}


bool 
dbDate::chars2ymd(const char* str, unsigned short& year, unsigned short& month, unsigned short& day, dateFieldOrder theOrder)
{
	istrstream s((char*)str);
	return istream2ymd(s, year, month, day, theOrder);
}


unsigned long 
dbDate::chars2Long(const char* str, dateFieldOrder theOrder)
{
	unsigned short year, month, day;
	istrstream s((char*)str);
	if (istream2ymd(s, year, month, day, theOrder))
		return ymd2Long(year, month, day);
	else
		return 0;
}


bool 
dbDate::checkDate(unsigned short year, unsigned short month, unsigned short day)
{
	if ( (day<1) || (month<1) || (year<1) )
		return false;
	if ( (day>31) || (month>12))
		return false;
// NOT YET IMPLEMENTED - proper date checking

	return true;
}

bool 
dbDate::istream2ymd(istream& is, unsigned short& year, unsigned short& month, unsigned short& day, char fieldSep, char recSep,
				dateFieldOrder theOrder)
{
//	if (is.peek()=='\n')
//		return false;  // early exit - empty line
	
	char ch;
	
	if (is.good()) {
		switch (theOrder) {
		case orderYMD :
			// skip non-digits
			skipTillDigit(is);
			// get year
			is >> year;
			// skip non-alphanumericals
			is >> ch;
			while (!isalnum(ch))
				is >> ch;
			is.putback(ch);
			// get month
			month = istream2month(is);
			// skip non-digits
			skipTillDigit(is);
			// get day
			is >> day;
			break;
				
		case orderDMY :
			// skip non-digits
			skipTillDigit(is);
			// get day
			is >> day;
			// skip post numerical characters (st,nd,rd,th)
			skipStreamPattern(is,"st");
			skipStreamPattern(is,"nd");
			skipStreamPattern(is,"rd");
			skipStreamPattern(is,"th");
			// skip non-alphanumericals
			is >> ch;
			while (!isalnum(ch))
				is >> ch;
			is.putback(ch);
			// get month
			month = istream2month(is);
			// get next character
			is >> ch;
			is.putback(ch);
			if ((ch==fieldSep)||(ch==recSep)) {
				is.putback(ch);
				year = currentYear();
				break;
			}
			// skip non-alphanumericals
			while (!isalnum(ch))
				is >> ch;
			is.putback(ch);
			// get year
			is >> year;
			break;
				
		case orderMDY :
			// skip non-alphanumericals
			is >> ch;
			while (!isalnum(ch))
				is >> ch;
			is.putback(ch);
			// get month
			month = istream2month(is);
			// skip non-digits
			skipTillDigit(is);
			// get day
			is >> day;
			// skip post numerical characters (st,nd,rd,th)
			skipStreamPattern(is,"st");
			skipStreamPattern(is,"nd");
			skipStreamPattern(is,"rd");
			skipStreamPattern(is,"th");
			// get next character
			is >> ch;
			if ((ch==fieldSep)||(ch==recSep)) {
				is.putback(ch);
				year = currentYear();
				break;
			}
			// skip non-alphanumericals
			while (!isalnum(ch))
				is >> ch;
			is.putback(ch);
			// get year
			is >> year;
			break;
		}
		
		if (year<100)
			year += currentCentury();
		return checkDate(year, month, day);
	}
	else
		return false;
}


bool 
dbDate::istream2ymd(istream& is, unsigned short& year, unsigned short& month, unsigned short& day, dateFieldOrder theOrder)
{
//	if (is.peek()=='\n')
//		return false;  // early exit - empty line
	
	char ch;
	
	if (is.good()) {
		switch (theOrder) {
		case orderYMD :
			// skip non-digits
			skipTillDigit(is);
			// get year
			is >> year;
			// skip non-alphanumericals
			is >> ch;
			while (!isalnum(ch))
				is >> ch;
			is.putback(ch);
			// get month
			month = istream2month(is);
			// skip non-digits
			skipTillDigit(is);
			// get day
			is >> day;
			break;
				
		case orderDMY :
			// skip non-digits
			skipTillDigit(is);
			// get day
			is >> day;
			// skip post numerical characters (st,nd,rd,th)
			skipStreamPattern(is,"st");
			skipStreamPattern(is,"nd");
			skipStreamPattern(is,"rd");
			skipStreamPattern(is,"th");
			// skip non-alphanumericals
			is >> ch;
			while (!isalnum(ch))
				is >> ch;
			is.putback(ch);
			// get month
			month = istream2month(is);
			// skip non-digits, check for eof
			do {
				is >> ch;
				if (!is.good()) {
					year = currentYear();
					break;
				}
			} while (!isdigit(ch));
			if (is.good()) {	
				is.putback(ch);
				// get year
				is >> year;
			}
			break;
				
		case orderMDY :
			// skip non-alphanumericals
			is >> ch;
			while (!isalnum(ch))
				is >> ch;
			is.putback(ch);
			// get month
			month = istream2month(is);
			// skip non-digits
			skipTillDigit(is);
			// get day
			is >> day;
			// skip non-digits, check for eof
			do {
				is >> ch;
				if (!is.good()) {
					year = currentYear();
					break;
				}	
			} while (!isdigit(ch));
			if (is.good()) {	
				is.putback(ch);
				// get year
				is >> year;
			}
			break;	
		}
		
		if (year<100)
			year += currentCentury();
		return checkDate(year, month, day);
	}
	else
		return false;
}


unsigned short
dbDate::istream2month(istream& is)
{
	char buf[4];
	char ch;
	unsigned short monthNum;
	is >> ch;
	// is month as character or digit?
	if (isalpha(ch)) {
		buf[0] = ch;
		is.get(buf[1]);
		is.get(buf[2]);
		buf[3]='\0';
		monthNum = chars2month(buf);
	}
	else {
		is.putback(ch);
		is >> monthNum;
	}
	// return month as digit
	return monthNum;
}


unsigned short
dbDate::chars2month (const char * s)
{
	char month[4];
	strncpy(month,s,3);
	month[3] = '\0';
	for (int i=0; month[i]; ++i)
		month[i]=toupper(month[i]);
	if (!strcmp(month,"JAN"))
		return 1;
	else if (!strcmp(month,"FEB"))
		return 2;
	else if (!strcmp(month,"MAR"))
		return 3;
	else if (!strcmp(month,"APR"))
		return 4;
	else if (!strcmp(month,"MAY"))
		return 5;
	else if (!strcmp(month,"JUN"))
		return 6;
	else if (!strcmp(month,"JUL"))
		return 7;
	else if (!strcmp(month,"AUG"))
		return 8;
	else if (!strcmp(month,"SEP"))
		return 9;
	else if (!strcmp(month,"OCT"))
		return 10;
	else if (!strcmp(month,"NOV"))
		return 11;
	else if (!strcmp(month,"DEC"))
		return 12;
	else
		return 0;	// error
}


unsigned long
dbDate::ymd2daynum( unsigned short y,unsigned short m,unsigned short d)
/*
Convert Gregorian calendar date to the corresponding Julian day number
j.  Algorithm 199 from Communications of the ACM, Volume 6, No. 8,
(Aug. 1963), p. 444.  Gregorian calendar started on Sep. 14, 1752.
This function not valid before that.
*/
{
	unsigned long c, ya;
	if (m > 2)
		m -= 3;
	else {
		m += 9;
		y--;
	} /* else */
	c = y / 100;
	ya = y - 100*c;
	return ((146097*c)>>2) + ((1461*ya)>>2) + (153*m + 2)/5 + d + 1721119;
}


void
dbDate::daynum2ymd(unsigned long dn,unsigned short& y,unsigned short& m,unsigned short& d)
/*
Convert a Julian day number to its corresponding Gregorian calendar
date.  Algorithm 199 from Communications of the ACM, Volume 6, No. 8,
(Aug. 1963), p. 444.  Gregorian calendar started on Sep. 14, 1752.
This function not valid before that.
*/
{
	unsigned long j = dn - 1721119;
	y = (unsigned short) (((j<<2) - 1) / 146097);
	j = (j<<2) - 1 - 146097*y;
	d = (unsigned short)(j>>2);
	j = ((d<<2) + 3) / 1461;
	d = (unsigned short)((d<<2) + 3 - 1461*j);
	d = (d + 4)>>2;
	m = (5*d - 3)/153;
	d = 5*d - 3 - 153*m;
	d = (d + 5)/5;
	y = (unsigned short)(100*y + j);
	if (m < 10)
		m += 3;
	else {
		m -= 9;
		y++;
	} /* else */
}


dbDate&
dbDate::operator++ ()
{
	// increment date
	setDaynum(daynum()+1);
	return *this;
}


dbDate&
dbDate::operator-- ()
{
	// decrement date
	setDaynum(daynum()-1);
	return *this;
}


dbDate&
dbDate::operator+= (int numDays)
{
	// add days to date
	setDaynum(daynum()+numDays);
	return *this;
}


dbDate&
dbDate::operator-= (int numDays)
{
	return (*this) += -numDays;
}


long
operator- (const dbDate& date1, const dbDate& date2)
{

	return date1.daynum() - date2.daynum();
}


unsigned long
dbDate::daynum() const
{
	unsigned short day, month, year;
	getDate(year, month, day);
	return ymd2daynum(year, month, day);
}


void
dbDate::setDaynum(unsigned long dn)
{
	unsigned short day, month, year;
	daynum2ymd(dn, year, month, day);
	setDate(year,month,day);
}


unsigned short
dbDate::day() const
{
	return ((unsigned long) *this) & 0xFF;
}


unsigned short
dbDate::month() const
{
	return (((unsigned long) *this) >> 8) & 0xFF;
}


unsigned short
dbDate::year() const
{
	return ((unsigned long) *this) >> 16;
}


unsigned short
dbDate::currentDay()
{
	unsigned short y,m,d;
	currentDate2ymd(y,m,d);
	return d;
}


unsigned short
dbDate::currentMonth()
{
	unsigned short y,m,d;
	currentDate2ymd(y,m,d);
	return m;
}

unsigned short
dbDate::currentYear()
{
	unsigned short y,m,d;
	currentDate2ymd(y,m,d);
	return y;
}


// -------------------------------------------------------
//                      d b R e a l
// -------------------------------------------------------
dbReal::dbReal()
{
	CompleteInit();
}


dbReal::dbReal(const OOF_IndexOptions indexOptions) :
				dbNumericField(indexOptions) 
{
	CompleteInit();
}


dbReal::dbReal(const char *fieldName)  :
				dbNumericField(fieldName)
{
	CompleteInit();
}


dbReal::dbReal(const char *fieldName, const OOF_IndexOptions indexOptions)  :
				dbNumericField(fieldName, indexOptions)  
{
	CompleteInit();
}


dbReal::dbReal(const dbReal& rhs) : dbNumericField(rhs) 
{
	CompleteInit();
	if (isStandalone())
		*((double*) mBackend)  = *((double*) rhs.mBackend) ;
}

 
void
dbReal::CompleteInit()
{
	if (isStandalone())
		mBackend = (OOF_tableBackend*) (new double);
}


dbReal::~dbReal()
{
}


void
dbReal::deleteStandaloneStorage()
{
	if (isStandalone())
		delete (double*) mBackend;
}


dbReal& 
dbReal::operator=(const dbReal& rhs)
{
	if (this == &rhs)
		return *this;
	
	*this = (double)rhs;
	return *this;
}


dbReal& 
dbReal::operator=(double n)
{
	if (isStandalone())
		*((double*) mBackend) = n;
	else {
		mTable->validateContext();
		double *writeTo = (double *)mBackend->getFieldWriteDest(mFieldNumber);
		if (writeTo)
			*writeTo = n;	
		mBackend->markDirty();
	}
	return *this;
}


void 
dbReal::setString(const char* str)
{
	double n;
	istrstream is((char*)str);  // cast away const for Borland
	is >> n;
	*this = n;
}


dbReal::operator double() const
{
	if (isStandalone()) 
		return *((double*) mBackend);
	else {
		mTable->validateContext();
		double *readFrom = (double *)mBackend->getFieldReadFrom(mFieldNumber);
		return *readFrom;
	}
}


void 
dbReal::copyValueFrom(const dbField* srcField)
{
	if (srcField->fieldType()==realField)
		*this = *(dbReal*)srcField;  // safe downcast
#ifdef OOF_Debug
	else
		dbConnect::raise(ostrstream() << "wrong field type passed in to dbReal::copyValueFrom, copying to " << fieldName() << " from " << srcField->fieldName());
#endif
}


OOF_fieldTypes 
dbReal::fieldType() const
{
	return realField; 
}


unsigned long 
dbReal::fieldLen() const
{
	return sizeof(double); 
}


void 
dbReal::extract(ostream& os) const
{
	os << (double)*this;
}


bool 
dbReal::insert(istream& is, char fieldSep, char recSep)
{
	// is the stream good?
	if (!is.good())
		return false;
	// read in the real
	double n;
	is >> n;
	// read in a potential field/record separator
	char c;
	is >> c;
	bool hitEndOfField = false;
	if ((c==fieldSep) || (c==recSep)) 
		hitEndOfField = true;
	// put back the potential field/record separator
	is.putback(c);
	if (hitEndOfField)
		*this = n;
	return hitEndOfField;
}


void 
dbReal::generateTestData(const char* /*testBuf*/, unsigned long /*testBufLen*/)
{
	*this = (1.7*rand()-rand()*1.5);
}


dbQueryTrinary	
dbReal::between(double fromNum, double toNum) const
{
	return dbQueryTrinary(new dbQueryField(this), dbQueryClause::between,
		new dbQueryLiteralDouble(fromNum), new dbQueryLiteralDouble(toNum));
}


dbQueryTrinary	
dbReal::outside(double fromNum, double toNum) const
{
	return dbQueryTrinary(new dbQueryField(this), dbQueryClause::outside,
		new dbQueryLiteralDouble(fromNum), new dbQueryLiteralDouble(toNum));
}


dbQueryBinary	
dbReal::operator==(double d) const
{
	return dbQueryBinary(new dbQueryField(this), dbQueryClause::equals, new dbQueryLiteralDouble(d));
}  


 dbQueryBinary	
 dbReal::operator!=(double d) const
{
	return dbQueryBinary(new dbQueryField(this), dbQueryClause::notEquals, new dbQueryLiteralDouble(d));
}  


 dbQueryBinary	
 dbReal::operator<(double d) const
{
	return dbQueryBinary(new dbQueryField(this), dbQueryClause::lessThan, new dbQueryLiteralDouble(d));
}  


 dbQueryBinary	
 dbReal::operator<=(double d) const
{
	return dbQueryBinary(new dbQueryField(this), dbQueryClause::lessThanOrEqual, new dbQueryLiteralDouble(d));
}  


dbQueryBinary	
dbReal::operator>(double d) const
{
	return dbQueryBinary(new dbQueryField(this), dbQueryClause::greaterThan, new dbQueryLiteralDouble(d));
}  


dbQueryBinary	
dbReal::operator>=(double d) const
{
	return dbQueryBinary(new dbQueryField(this), dbQueryClause::greaterThanOrEqual, new dbQueryLiteralDouble(d));
}  

 
dbQueryLiteral* 
dbReal::MakeQueryLiteral(long n) const
{
	return new dbQueryLiteralDouble(n);
}



void
skipStreamPattern(istream& is, const char * str)
{
	char * buf;
	
	// get pattern length
	unsigned short i;
	for (i=0; str[i]; ++i)
		;
	buf = new char[i + 1];
	unsigned short j;
	for (j=0; j < i; ++j)
		is >> buf[j];
	buf[j] = '\0';
	if (strcmp(buf,str))
		for (int k = (j-1); k >= 0; --k)
			is.putback(buf[k]);
	delete [] buf;
	return;
}
