// oofile.hpp

// This is the only file you need to include in a user program to use the database
// if you are using the report-writer, you also need oofrw.hpp

// Backend database file - change if using other backends
#include "oofctree.hpp"


// basic OOFILE files
#include "oof4.hpp" 
#include "oofquery.hpp" 
#include "oofrel.hpp" 
#include "oofview.hpp" 

// mainly used in demo progs
#ifndef EXIT_SUCCESS
  #define EXIT_SUCCESS 0
#endif


// macros that define user files

// define macros for table construction
#define DECLARE_REF(tb) \
class tb; \
class tb ## Ref : public dbRelRef \
{ public: tb* operator->() { return (tb*) relatedTable();}; }; 


#define DECLARE_SET(tb) \
class tb; \
class tb ## Set : public dbRelSet \
{ public: tb* operator->() { return (tb*) relatedTable();}; }; 

#define COMMON_METHODS(tb) \
public: \
virtual const char* defaultName() const { return #tb ;}; \
tb&	operator[](const dbQueryClause& rhs) { search(rhs); return *this; }; \
tb&	operator[](unsigned long index) {	gotoRecord(index); return *this;}; \
tb&	operator[](long index) {	assert(index>=0); gotoRecord(index); return *this;}; \
virtual dbTable* cloneWithoutSelection() { return clone(true); }; \
tb& operator-(const tb& rhs) { dbTable::difference_with(rhs); return *this; }; \
tb& difference_with(const tb& rhs) { dbTable::difference_with(rhs); return *this; }; \
tb& operator&(const tb& rhs) { dbTable::intersection_with(rhs); return *this; }; \
tb& intersection_with(const tb& rhs) { dbTable::intersection_with(rhs); return *this; }; \
tb& operator|(const tb& rhs) { dbTable::union_with(rhs); return *this; }; \
tb& operator+(const tb& rhs) { dbTable::union_with(rhs); return *this; }; \
tb& union_with(const tb& rhs) { dbTable::union_with(rhs); return *this; }; \
friend ostream& operator<<(ostream& os, tb& tbl) {tbl.extract(os); return os;}; \
friend ostream& operator<<(ostream& os, tb* tbl) {tbl->extract(os); return os;}; \
friend istream& operator>>(istream& is, tb& tbl) {tbl.insert(is); return is;}; \
friend istream& operator>>(istream& is, tb* tbl) {tbl->insert(is); return is;}; 

//tb&	operator[](const char *name) {	searchDefaultIndex(name); return *this;}; \
//tb* operator&() { assert(0);  return this;}; \
//tb* invert(const dbTablePtr& rhs) { AddHalfRelation((dbTable**) &rhs); return this; }; \


#define OOFILE_METHODS(tb) \
COMMON_METHODS(tb) \
virtual dbTable* clone(bool withoutSelection=false) {  \
	sCloningWithoutSelection = withoutSelection; \
	sCloning = true; \
	tb* ret = new tb(*this); \
	ret->postCloneCleanup(); \
	return ret; };

#define ABSTRACT_BASE_METHODS(tb) \
COMMON_METHODS(tb) \
virtual dbTable* clone(bool withoutSelection=false)=0; 



#define DECLARE_ABSTRACT_CLASS(tb) \
class tb : public dbTable { \
ABSTRACT_BASE_METHODS(tb) 


#define DECLARE_CLASS(tb) \
class tb : public dbTable { \
OOFILE_METHODS(tb) 

// backwards compatability with old macro names
#define CLASS_TABLE DECLARE_CLASS
#define REF_TABLE DECLARE_REF
#define SET_TABLE DECLARE_SET
