// COPYRIGHT 1996 A.D. Software, All rights reserved

// messaging layer of OOFILE database

#include "oof2.hpp"

// -------------------------------------------------------
//                      d b B r o a d c a s t e r 
// -------------------------------------------------------
dbBroadcaster::~dbBroadcaster()
{
	if (mReceivers) {
		broadcast(OOFmsg_BroadcasterClosing);
		delete mReceivers;
	}
}


void 
dbBroadcaster::broadcast(OOFmsgT msg, unsigned long senderDefined)
{
	if (mReceivers) {
		unsigned short numReceivers = mReceivers->count();
		for (unsigned short i=0; i<numReceivers; i++) {
			dbReceiver* rcvr = (dbReceiver*) (mReceivers->item(i));  // safe downcast
			rcvr->receiveMsg(msg, senderDefined);
		}
	}
}


void 
dbBroadcaster::addReceiver(dbReceiver* r)
{
	if (!mReceivers) {
		 mReceivers = new OOF_ExpandableLongArray(1);
		 assert(mReceivers);
	}
	mReceivers->append((unsigned long) r);
}


void 
dbBroadcaster::removeReceiver(const dbReceiver* r)
{
	if (mReceivers)
		mReceivers->deleteMatching((unsigned long) r);
}


// -------------------------------------------------------
//                      d b R e c e i v e r
// -------------------------------------------------------
void
dbReceiver::subscribeTo(dbBroadcaster* broad)
{
	broad->addReceiver(this);
}


void
dbReceiver::unsubscribe(dbBroadcaster* broad)
{
	broad->removeReceiver(this);
}




// -------------------------------------------------------
//         d b S i n g l e L i s t e n e r
// -------------------------------------------------------
dbSingleListener::dbSingleListener(dbBroadcaster* listensTo) :
		mListensTo(listensTo),
		mSuspendedListensTo(0),
		mSuspendCount(0)
{
	if (listensTo)
		listensTo->addReceiver(this);
}


dbSingleListener::dbSingleListener(const dbSingleListener& rhs) :
		mListensTo(rhs.mListensTo),
		mSuspendedListensTo(rhs.mSuspendedListensTo),
		mSuspendCount(rhs.mSuspendCount)
{
	if (mListensTo)
		mListensTo->addReceiver(this);
}


void 
dbSingleListener::receiveMsg(OOFmsgT msg, unsigned long /*senderDefined*/)
{
	switch (msg) {
	
	case (OOFmsg_BroadcasterClosing) : 
		mListensTo = 0;
		break;
	};
}


dbSingleListener::~dbSingleListener() 
{
	unsubscribe();
} 


void
dbSingleListener::subscribeTo(dbBroadcaster* broad)
{
	mListensTo = broad;
	broad->addReceiver(this);
}


void
dbSingleListener::unsubscribe(dbBroadcaster* broad)
{
	if (broad) {
		assert(broad==mListensTo);
		;
	}
	if (mListensTo)
		dbReceiver::unsubscribe(mListensTo);
	mListensTo = 0;
}


void
dbSingleListener::suspendListening()
{
	mSuspendCount++;
	if (mSuspendCount==1) {
		mSuspendedListensTo = mListensTo;
		unsubscribe();
	}
	else {
		assert(mSuspendedListensTo);
#ifdef OOF_Debug
// sanity check on nested suspends
		if (mSuspendCount > 1000) {
			dbConnect::raise("You've nested more than 1000 suspend listenings");
		}
#endif
		;
	}
}


void
dbSingleListener::resumeListening()
{
	if (mSuspendedListensTo==0)
		return; // let them get away with being sloppy about extra resumes
	assert(mSuspendCount>0);
	mSuspendCount--;
	if (mSuspendCount==0) {
		subscribeTo(mSuspendedListensTo);
		mSuspendedListensTo = 0;
	}
}
  