// COPYRIGHT 1994 A.D. Software, All rights reserved

// OOFILE database queries

#include "oofquery.hpp"
#include "oof3.hpp"
#include <assert.h>

// Note: these consts are for future upgrades
char dbQueryClause::sWildcardMultiple = '*';
char dbQueryClause::sWildcardSingle = '?';


// -------------------------------------------------------
//                 d b Q u e r y C l a u s e
// -------------------------------------------------------
bool
dbQueryClause::wildcardsInString(const char* schStr) 
{
	for(int i=0;schStr[i];i++) 
		if((schStr[i]==sWildcardMultiple)||(schStr[i]==sWildcardSingle))
			return true;
	return false;
}


bool 
dbQueryClause::isSimpleIndexedSearch() const
{
	return false;
}

 
dbQueryBinaryCombo 
dbQueryClause::operator&(const dbQueryClause& rhs) const
{
	return dbQueryBinaryCombo(this, dbQueryClause::and, &rhs);
}


dbQueryBinaryCombo 
dbQueryClause::operator&&(const dbQueryClause& rhs) const
{
	return dbQueryBinaryCombo(this, dbQueryClause::and, &rhs);
}


dbQueryBinaryCombo 
dbQueryClause::operator|(const dbQueryClause& rhs) const
{
	return dbQueryBinaryCombo(this, dbQueryClause::or, &rhs);
}


dbQueryBinaryCombo 
dbQueryClause::operator||(const dbQueryClause& rhs) const
{
	return dbQueryBinaryCombo(this, dbQueryClause::or, &rhs);
}


dbQueryBinaryComboOwner*
dbQueryClause::operator|(dbQueryClause* adoptRHS) 
{
   return new dbQueryBinaryComboOwner(this, dbQueryClause::or, adoptRHS);
}


dbQueryBinaryComboOwner*
dbQueryClause::operator||(dbQueryClause* adoptRHS) 
{
   return new dbQueryBinaryComboOwner(this, dbQueryClause::or, adoptRHS);
}


dbQueryBinaryComboOwner*
dbQueryClause::operator&(dbQueryClause* adoptRHS)
{
   return new dbQueryBinaryComboOwner(this, dbQueryClause::and,adoptRHS);
}


dbQueryBinaryComboOwner*
dbQueryClause::operator&&(dbQueryClause* adoptRHS)
{
   return new dbQueryBinaryComboOwner(this, dbQueryClause::and,adoptRHS);
}


// -------------------------------------------------------
//                 d b Q u e r y 
// -------------------------------------------------------
dbQuery::dbQuery() : mQuery(0)
{
}


dbQuery::~dbQuery()
{
	delete mQuery;
}



dbQueryClause::QueryClauseTypes  
dbQuery::queryClauseType()  const
{
	return builtQuery;
}


void
dbQuery::operator|=(dbQueryClause* adoptedBit) 
{
   if (mQuery)
      mQuery = *mQuery || adoptedBit;
   else
      mQuery = adoptedBit;
}


void
dbQuery::operator&=(dbQueryClause* adoptedBit) 
{
   if (mQuery)
      mQuery = *mQuery && adoptedBit;
   else
      mQuery = adoptedBit;
}


// -------------------------------------------------------
//                 d b Q u e r y B i n a r y 
// -------------------------------------------------------
dbQueryBinary::dbQueryBinary(dbQueryField* fld, BinaryQueryOps op, dbQueryClause* litOrFld) :
				mLhs(fld),
				mRhs(litOrFld),
				mBinOp(op)
{
	assert(fld->field() && !fld->field()->isStandalone());
}
 
 
dbQueryBinary::dbQueryBinary(dbQueryBinary& rhs) :
	mLhs (rhs.mLhs),
	mRhs(rhs.mRhs),
	mBinOp(rhs.mBinOp)
{
	rhs.mLhs = 0;
	rhs.mRhs = 0;
}


dbQueryBinary::~dbQueryBinary()
{
	delete mLhs;
	delete mRhs;
}


dbQueryBinary& 
dbQueryBinary::operator=(dbQueryBinary&) 
{ 
	assert(0);  // should never be called
	return *this; 
};


dbQueryClause::QueryClauseTypes  
dbQueryBinary::queryClauseType()  const
{
	if (mRhs->queryClauseType()==atomicfield)
		return binaryfieldTofield;
	else
		return binaryfieldToLiteral;
}


dbQueryLiteralStr* 
dbQueryBinary::literalStrClause() const
{
	if (mRhs->queryClauseType()==atomicLiteralStr) 
		return (dbQueryLiteralStr*) mRhs;  // safe downcast
	else
		return 0;
}


dbQueryLiteral* 
dbQueryBinary::literalClause() const
{
// NOTE the following assumes that logic to handle binary logical clauses is a superset of
// that handling strings, and if the difference is important we have tried literalStrClause()
// BEFORE trying this routine
	if ((mRhs->queryClauseType()==atomicLiteral) || (mRhs->queryClauseType()==atomicLiteralStr))
		return (dbQueryLiteral*) mRhs;  // safe downcast, actually to a parent of the exact type (a dbQueryLiteralLong etc.)
	else
		return 0;
}


bool 
dbQueryBinary::isSimpleIndexedSearch() const
{
	if (queryClauseType() != binaryfieldToLiteral)
		return false;

	const dbField* schField = lhsField();
	if (!schField->fieldIsIndexed())
		return false;
		
	if (schField->fieldType() != charField)
		return true;  // binary indexed field!
	
	if (binaryOperator() != equals && 
		binaryOperator() != startsWith && 
		binaryOperator() != notEquals 
	)
		return true;  // indexed and can't be wildcard

	dbQueryLiteralStr* strRHS = dbQueryBinary::literalStrClause();
	assert(strRHS);  // as lhs is char field
	return strRHS->hasWildcards();
}

 
// -------------------------------------------------------
//           d b Q u e r y B i n a r y C o m b o
// -------------------------------------------------------
dbQueryBinaryCombo::dbQueryBinaryCombo(const dbQueryClause* lhs, QueryCombinatorialOps op, const dbQueryClause* rhs) :
				mLhs(lhs),
				mRhs(rhs),
				mComboOp(op),
				mTempKeyStoreToDispose(0),
				mTempKeyStoreToDispose2(0)
				{}
				
				
 dbQueryBinaryCombo::~dbQueryBinaryCombo()
{
	delete[] mTempKeyStoreToDispose;
	delete[] mTempKeyStoreToDispose2;
}


dbQueryBinaryCombo::dbQueryBinaryCombo(const dbQueryBinaryCombo& rhs) :
				mLhs(rhs.mLhs),
				mRhs(rhs.mRhs),
				mComboOp(rhs.mComboOp),
				mTempKeyStoreToDispose(0),
				mTempKeyStoreToDispose2(0)
{}


dbQueryBinaryCombo& 
dbQueryBinaryCombo::operator=(dbQueryBinaryCombo&) 
{ 
	assert(0);  // should never be called
	return *this; 
};


dbQueryClause::QueryClauseTypes  
dbQueryBinaryCombo::queryClauseType()  const
{
	return binaryCombination;
}

	
const dbQueryClause* 
dbQueryBinaryCombo::item(unsigned int i) const
{
	assert(i<2);  // for now, later will have a list of items
	if (i==0)
		return mLhs;
	else
		return mRhs;
}


unsigned long 
dbQueryBinaryCombo::pairFieldsIfCouldUseCompoundIndex() const
{
	if (mComboOp != and)
		return 0;  // early failure - they're not ANDed
		
	if (mLhs->queryClauseType() != binaryfieldToLiteral)
		return 0;
		
	if (mRhs->queryClauseType() != binaryfieldToLiteral)
		return 0;
		
	dbQueryBinary* lhs = (dbQueryBinary*) mLhs;  // safe downcast due to above tests 
	if (lhs->binaryOperator() != equals)
		return 0;
		
	dbQueryBinary* rhs = (dbQueryBinary*) mRhs;  // safe downcast due to above tests 
	if (rhs->binaryOperator() == notEquals) 
		return 0;
		
// if get here have an expression that CAN be used with a compound index
// ie:  A==a && B==b -> == ab
// A==a && B.startsWith(b) -> startsWith(ab)
// A==a && B>=b -> between(ab, a+)

	const dbField* fld = lhs->lhsField();
	unsigned long ret = fld->fieldNumber();
	fld = rhs->lhsField();
	ret = (ret << 16) + fld->fieldNumber();
	return ret;
}


dbQueryClause* 
dbQueryBinaryCombo::makeCompoundSearch(const dbCompoundField* fld)
{
// this gets a little complicated:
// we make this look like a string query, BUT 
// either of our components can really be binary values
// and we also must pad out the LHS to its full key width

// WARNING this is only partially implemented, mainly for 2nd being > dbDate
	assert(fld->fieldLen()<USHRT_MAX);
	unsigned short keyLen = fld->fieldLen();
	unsigned short firstFieldLen = fld->field(0)->fieldLen();
	char* target = new char[keyLen];  // uh oh - we need to delete this after the query succeeds!
	assert(target);
	delete[] mTempKeyStoreToDispose;  // just in case we are reusing a query object somehow
	delete[] mTempKeyStoreToDispose2;
	mTempKeyStoreToDispose = 0;
	mTempKeyStoreToDispose2 = 0;
	
	assert(mLhs->queryClauseType() == binaryfieldToLiteral);
	dbQueryLiteral* lit = ((dbQueryBinary*) mLhs)->literalClause();  // safe downcast
	assert(lit);
	memset(target, 0, keyLen);  // clear the entire compound key width
// NOT YET IMPLEMENTED - would be a loop for more than 2 segments
	memcpy(target, lit->binaryContents(), lit->literalLen());	
	#ifdef OOF_SmartHeap
		assert(MemPoolCheck(MemDefaultPool));
	#endif	
	
	assert(mRhs->queryClauseType() == binaryfieldToLiteral);
	dbQueryBinary* rhs = (dbQueryBinary*) mRhs;  // safe downcast
	dbQueryLiteral* finalLit = rhs->literalClause();
	
	BinaryQueryOps theOp = rhs->binaryOperator();
	if (fld->segment(1)->isReversed()) // need to swap the sense of some ops
		switch(theOp) {
		case greaterThan:
			theOp = lessThan;
			break;
			
		case greaterThanOrEqual:
			theOp = lessThanOrEqual;
			break;
			
		case lessThan:
			theOp = greaterThan;
			break;
			
		case lessThanOrEqual:
			theOp = greaterThanOrEqual;
			break;
			
		default:
			assert(theOp==equals || theOp==startsWith);
			break;
		}
		
	switch(theOp) {
	case greaterThan:
	case greaterThanOrEqual:
		mTempKeyStoreToDispose2 = target;  // and fill 2nd field with 0xFF
		target += firstFieldLen;
		memset(target, 0xFF, keyLen-firstFieldLen);  
		
		target = new char[keyLen]; 
		assert(target);
		mTempKeyStoreToDispose = target;
		memset(target, 0, keyLen);  // clear the entire compound key width
	// NOT YET IMPLEMENTED - would be a loop for more than 2 segments
		memcpy(target, lit->binaryContents(), lit->literalLen());	
		#ifdef OOF_SmartHeap
			assert(MemPoolCheck(MemDefaultPool));
		#endif
		target += firstFieldLen;
		memcpy(target, finalLit->binaryContents(), finalLit->literalLen());	
		if (theOp == greaterThan)  // HORRIBLE HACK THAT ONLY WORKS ON DATES
			mTempKeyStoreToDispose2[keyLen-1] = 0xFF;  // EFFECTIVE INCREMENT may now work on int, real etc.
		return new dbQueryTrinary(new dbQueryField(fld), dbQueryClause::between,
			new dbQueryLiteralBLOB(mTempKeyStoreToDispose, keyLen), 
			new dbQueryLiteralBLOB(mTempKeyStoreToDispose2, keyLen));
		break;
		
		
	default:  // for now
		mTempKeyStoreToDispose = target;
		target += firstFieldLen;
		memcpy(target, finalLit->binaryContents(), finalLit->literalLen());	
		#ifdef OOF_SmartHeap
			assert(MemPoolCheck(MemDefaultPool));
		#endif
		return new dbQueryBinary(new dbQueryField(fld), 
			rhs->binaryOperator(), 
			new dbQueryLiteralBLOB(mTempKeyStoreToDispose, keyLen)
		);
	}
}



// -------------------------------------------------------
//       d b Q u e r y B i n a r y C o m b o O w n e r
// -------------------------------------------------------
dbQueryBinaryComboOwner::dbQueryBinaryComboOwner(dbQueryClause* lhs, QueryCombinatorialOps op, dbQueryClause* rhs) :
				dbQueryBinaryCombo(lhs, op, rhs),
				mOurLhs(lhs),
				mOurRhs(rhs)
				{}
				
				
dbQueryBinaryComboOwner::~dbQueryBinaryComboOwner()
{
	delete mOurLhs;
	delete mOurRhs;
}


// -------------------------------------------------------
//                 d b Q u e r y T r i n a r y 
// -------------------------------------------------------
// NOTE there is a lot of horrible code in this class
// it is offensively coarse, but reasonably efficient and a long way
// down the priority list for a rewrite.
dbQueryTrinary::dbQueryTrinary(dbQueryTrinary& rhs)
{
	mLhs = rhs.mLhs;
	mFrom = rhs.mFrom;
	mTo = rhs.mTo;
	mTrinOp = rhs.mTrinOp;
	rhs.mLhs = 0;
	rhs.mFrom = 0;
	rhs.mTo = 0;
}


dbQueryTrinary::~dbQueryTrinary()
{
	delete mLhs;
	delete mFrom;
	delete mTo;
}


dbQueryTrinary& 
dbQueryTrinary::operator=(dbQueryTrinary&) 
{ 
	assert(0);  // should never be called
	return *this; 
};


dbQueryClause::QueryClauseTypes  
dbQueryTrinary::queryClauseType()  const
{
		return trinaryFieldToLiterals;
}


dbQueryLiteralStr* 
dbQueryTrinary::literalStrFromClause() const
{
	if (mFrom->queryClauseType()==atomicLiteralStr) 
		return (dbQueryLiteralStr*) mFrom;  // safe downcast
	else
		return 0;
}


dbQueryLiteralStr* 
dbQueryTrinary::literalStrToClause() const
{
	if (mTo->queryClauseType()==atomicLiteralStr) 
		return (dbQueryLiteralStr*) mTo;  // safe downcast
	else
		return 0;
}


dbQueryLiteral* 
dbQueryTrinary::literalFromClause() const
{
	if (mFrom->queryClauseType()==atomicLiteral) 
		return (dbQueryLiteral*) mFrom;  // safe downcast
		return 0;
}

	
dbQueryLiteral* 
dbQueryTrinary::literalToClause() const
{
	if (mTo->queryClauseType()==atomicLiteral) 
		return (dbQueryLiteral*) mTo;  // safe downcast
	else
		return 0;
}

	
// -------------------------------------------------------
//                 d b Q u e r y F i e l d
// -------------------------------------------------------
dbQueryClause::QueryClauseTypes  
dbQueryField::queryClauseType()  const
{
	return atomicfield;
}


// -------------------------------------------------------
//                 d b Q u e r y L i t e r a l
// -------------------------------------------------------
dbQueryClause::QueryClauseTypes  
dbQueryLiteral::queryClauseType()  const
{
	return atomicLiteral;
}


const void* 
dbQueryLiteral:: binaryContents() const
{
	return 0;
}


unsigned short 
dbQueryLiteral::literalLen() const
{
	return 0;
}

	
// -------------------------------------------------------
//          d b Q u e r y L i t e r a l L o n g
// -------------------------------------------------------
const void* 
dbQueryLiteralLong:: binaryContents() const
{
	return &mNum;
}


unsigned short 
dbQueryLiteralLong::literalLen() const
{
	return sizeof(long);
}


	
// -------------------------------------------------------
//          d b Q u e r y L i t e r a l U l o n g
// -------------------------------------------------------
const void* 
dbQueryLiteralUlong:: binaryContents() const
{
	return &mNum;
}


unsigned short 
dbQueryLiteralUlong::literalLen() const
{
	return sizeof(unsigned long);
}


	
// -------------------------------------------------------
//          d b Q u e r y L i t e r a l S h o r t
// -------------------------------------------------------
const void* 
dbQueryLiteralShort:: binaryContents() const
{
	return &mNum;
}


unsigned short 
dbQueryLiteralShort::literalLen() const
{
	return sizeof(short);
}


	
// -------------------------------------------------------
//          d b Q u e r y L i t e r a l U s h o r t
// -------------------------------------------------------
const void* 
dbQueryLiteralUshort:: binaryContents() const
{
	return &mNum;
}


unsigned short 
dbQueryLiteralUshort::literalLen() const
{
	return sizeof(unsigned short);
}


	
// -------------------------------------------------------
//         d b Q u e r y L i t e r a l D o u b l e
// -------------------------------------------------------
const void* 
dbQueryLiteralDouble:: binaryContents() const
{
	return &mNum;
}


unsigned short 
dbQueryLiteralDouble::literalLen() const
{
	return sizeof(double);
}


	
// -------------------------------------------------------
//         d b Q u e r y L i t e r a l B L O B
// -------------------------------------------------------
const void* 
dbQueryLiteralBLOB:: binaryContents() const
{
	return mBLOB;
}


unsigned short 
dbQueryLiteralBLOB::literalLen() const
{
	return mBLOBlen;
}


	
// -------------------------------------------------------
//            d b Q u e r y L i t e r a l S t r
// -------------------------------------------------------
dbQueryClause::QueryClauseTypes  
dbQueryLiteralStr::queryClauseType()  const
{
	return atomicLiteralStr;
}

 
const void* 
dbQueryLiteralStr::binaryContents() const
{
	return mStr;
}


unsigned short 
dbQueryLiteralStr::literalLen() const
{
	if (!mStr)
		return 0;
	return strlen(mStr);
}


bool
dbQueryLiteralStr::hasWildcards()
{
	if (mCheckedForWildcards == neverChecked) {		
		assert(mStr);
		if (dbQueryClause::wildcardsInString(mStr))
			mCheckedForWildcards = hasWild;
		else
			mCheckedForWildcards = hasNone;
	}
	return (mCheckedForWildcards == hasWild);
	
}