#ifndef H_OOFREL
#define H_OOFREL
// COPYRIGHT 1994 A.D. Software, All rights reserved

// public layer of OOFILE database

#include "oof2.hpp"

class dbRelRefBase : public dbField, public dbSingleListener
{
public:
	dbRelRefBase(unsigned long maxLinks=0);
	dbRelRefBase(const dbRelRefBase& rhs);
	virtual ~dbRelRefBase() {};
	
	virtual void postCloneCleanup();
	void completeCloneOfInverse(dbRelRefBase* cloner);
	
// finish specifying the relationship
	void relatesTo(dbRelRefBase&);
	
	void joinField(dbField&);
	dbField* joinField() const;
	bool propagatesDeletes() const;
	void propagateRelatedDeletes(bool prop=true);
	
// mandatory dbField overrides
	virtual OOF_fieldTypes fieldType() const;
	virtual unsigned long fieldLen() const;
	
// reflective operations
	virtual const OOF_String& fieldName() const;
	virtual void describe(ostream&);
	virtual void extract(ostream&) const;
	virtual OOF_fieldTypes nativeType() const;
	virtual bool fieldIsVirtual() const;
	dbTable* relatedTable();
	
// perform the relationship traversal
	void relateRecord();
	void relateSelection();
	void relateFromRecord();

// update link for rhs
	void updateRelField();
	
	virtual void receiveMsg(OOFmsgT msg, unsigned long senderDefined);

protected:
	void	BuildRelatedTable();
	bool CompletelySpecified() const;
	void SetCommonRelationshipFields(dbRelRefBase&);
	dbRelRefBase* OurInverse() const;
	oidT OID() const;
	void OID(oidT);

// data storage
	unsigned long mMinLinks, mMaxLinks;
	dbTable* mRelatedTableProto;  // the prototype of the related table
	dbTable* mActualRelatedTable;  // the cloned table managing the related selection
	fieldNumT	mInverseFieldNumber;
	dbField* mJoinField;
	bool mPropagatesDeletes;
	dbRelRefBase* mInverseField;
};


class dbRelRef : public dbRelRefBase
{
public:
	dbRelRef() : dbRelRefBase(1 /* only 1 related value allowed */) {};
	// use default copy ctor
};


class dbRelSet : public dbRelRefBase
{
public:
	dbRelSet() : dbRelRefBase() {};
	// use default copy ctor
};


class dbRelationship
{
public:	
// just exists to declare a relationship
	dbRelationship(dbRelRefBase&, dbRelRefBase&);
	dbRelationship(dbRelRefBase*, dbRelRefBase*);
// variations with join fields
	dbRelationship(dbRelRefBase&, dbRelRefBase&, dbField&, dbField&);
	dbRelationship(dbRelRefBase*, dbRelRefBase*, dbField*, dbField*);

};


// include inline definitions
#include "oofrel.inl"
#endif