// COPYRIGHT 1994 A.D. Software, All rights reserved

// public layer of OOFILE database - database views

#include "oof3.hpp"  // knows a bit about fields
#include "oofrel.hpp"  // knows about relations
#include "oofview.hpp"

// -------------------------------------------------------
//                     d b V i e w
// -------------------------------------------------------

dbView::dbView(dbTable*	tbl)
{
	mSource = new dbSourceTable(tbl);
	assert(mSource);
}


dbView::dbView(dbTable&	tbl)
{
	mSource = new dbSourceTable(&tbl);
	assert(mSource);
}


dbView::dbView(dbRelRefBase* tblRef)
{
	mSource = new dbSourceTable(tblRef->relatedTable());
	assert(mSource);
}


dbView::dbView(dbRelRefBase& tblRef)
{
	mSource = new dbSourceTable(tblRef.relatedTable());
	assert(mSource);
}


dbView::dbView(const dbView& rhs) : 
				OOF_Dictionary(rhs)
{
	mSource = rhs.mSource->clone();
}


const dbView& 
dbView::operator=(const dbView& rhs)
{
	if (this == &rhs)
		return *this;
	
	delete mSource;
	mSource = rhs.mSource->clone();
	return *this;
}


dbView::~dbView()
{
	delete mSource;
}


dbField& 
dbView::field(unsigned int fieldNoInView)
{
	dbField* theField = (dbField*) item(fieldNoInView);  // safe downcast
	return *theField;
}


ostream& 
operator<<(ostream& os, dbView& theView)
{
	theView.extract(os);
	return os;
}


void 
dbView::extract(ostream& os)
{
	source()->start();
	unsigned long numRecs = source()->count();
	unsigned int numFields = count();
	for (unsigned long row=0; row<numRecs; row++) {	// iterate through records (vertical)
		for (unsigned int col=0; col<numFields; col++) {		// start field iteration (horizontal)
			dbField  *theField = (dbField *) value(col); 		  // safe downcast
			theField->extract(os);
			os << '\t';
		}
		os << endl;
		if (row<numRecs-1)
			source()->next();
	}
}



// -------------------------------------------------------
//               d b S o u r c e T a b l e 
// -------------------------------------------------------
dbSource* 
dbSourceTable::clone() const
{
	return new dbSourceTable(*this);
}


void 
dbSourceTable::next()
{
	mTable->next();
}


unsigned long 
dbSourceTable::count()
{
	return mTable->count();
}


void 
dbSourceTable::saveRecord()
{
	mTable->saveRecord();
}


void 
dbSourceTable::newRecord()
{
	mTable->newRecord();
}


bool 
dbSourceTable::gotoRecord(unsigned long recNo)
{
	return mTable->gotoRecord(recNo);
}
