// CPhoneControlApp.cp -- application methods
// Created 01/01/95 12:01 PM by AppMaker

#include "CPhoneControlApp.h"

#include "CPhoneControlDoc.h"
#include "CMainWindow.h"
#include "CAboutDialog.h"
#include "CImporting.h"
#include "CCompletedImport.h"

#include <LFile.h>
#include <UDesktop.h>
#include <URegistrar.h>
#include <UScreenPort.h>
#include <PPobClasses.h>
#include <PP_Messages.h>
#include <StandardFile.h>

#include "oof1.hpp"
#include "oofpp.hpp"

#include "LControlEnablingEditField.h"
#include "LStrTableView.h"

#ifdef BLIND_OPEN
	char CPhoneControlApp::sDatabaseLocationPath[256];
	char CPhoneControlApp::sImportFilePath[256];
	Boolean CPhoneControlApp::sImporting = false;
	DateTimeRec CPhoneControlApp::sStartImport;
	unsigned long CPhoneControlApp::sImportInterval;
#endif

// ---------------------------------------------------------------------------
//		 CPhoneControlApp
// ---------------------------------------------------------------------------
//	Default constructor

CPhoneControlApp::CPhoneControlApp()
	:LDocApplication()
{
	UScreenPort::Initialize();

	RegisterClasses();

	SetUpMenus();

	// initialize app's data:

}


// ---------------------------------------------------------------------------
//		 ~CPhoneControlApp
// ---------------------------------------------------------------------------
//	Destructor
//

CPhoneControlApp::~CPhoneControlApp()
{
}

// ---------------------------------------------------------------------------
//		 RegisterClasses
// ---------------------------------------------------------------------------
//

void
CPhoneControlApp::RegisterClasses()
{
	RegisterAllPPClasses();
			// replace to register only classes that we use.
			// windows know what classes they use
			// so call static functions in each window/dialog,
			// or generate a single function as union of all w/d's.

	// register custom pane classes
	URegistrar::RegisterClass('Maiw', (ClassCreatorFunc)CMainWindow::CreateMainWindowStream);
	URegistrar::RegisterClass('Abog', (ClassCreatorFunc)CAboutDialog::CreateAboutDialogStream);
	URegistrar::RegisterClass('Impg', (ClassCreatorFunc)CImporting::CreateImportingStream);
	URegistrar::RegisterClass('Comt', (ClassCreatorFunc)CCompletedImport::CreateCompletedImportStream);
	URegistrar::RegisterClass(LControlEnablingEditField::class_ID, (ClassCreatorFunc)LControlEnablingEditField::CreateControlEnablingEditFieldStream);
	URegistrar::RegisterClass(LStrTableView::class_ID, (ClassCreatorFunc)LStrTableView::CreateStrTableViewStream);
}


// ---------------------------------------------------------------------------
//		 SetUpMenus
// ---------------------------------------------------------------------------
//

void
CPhoneControlApp::SetUpMenus()
{

}

// ---------------------------------------------------------------------------
//		 StartUp
// ---------------------------------------------------------------------------
//	The application calls this member function automatically when the application
//	is opened

void
CPhoneControlApp::StartUp()
{
	ObeyCommand(cmd_Open, nil);
}

	//----------
	LModelObject*
	CPhoneControlApp::MakeNewDocument()
	{
	CPhoneControlDoc	*theDoc = new CPhoneControlDoc(this);

	theDoc->newFile();

	return theDoc;
}

//----------
// called from SendAEOpenDoc in response to Open command

void
CPhoneControlApp::OpenDocument(
	FSSpec	*inMacFSSpec)
{
	CPhoneControlDoc	*theDoc = new CPhoneControlDoc(this);

	theDoc->openFile (inMacFSSpec);
}

//----------
// called from LDocApplication in response to Open command

void
CPhoneControlApp::ChooseDocument()
{
#ifdef BLIND_OPEN
	if (dbConnect::fileExists("PhoneControl Database Location")) {
		ifstream locnFile("PhoneControl Database Location");
		locnFile.getline(sDatabaseLocationPath, sizeof(sDatabaseLocationPath));
		locnFile.close();
		
		if (dbConnect::fileExists("PhoneControl Import Location")) {
			ifstream impFile("PhoneControl Import Location");
			impFile.getline(sImportFilePath, sizeof(sImportFilePath));
			sImporting = true;
// need to get time!
			GetTime(&sStartImport);
			impFile >> sStartImport.hour >> sStartImport.minute;  // could do with some error checking
			sImportInterval = 0;
			if (!impFile.eof())
				impFile >> sImportInterval;
			if (sImportInterval==0)
				sImportInterval = 24 * 60 * 60;  // 24 hours
			impFile.close();
		}
	}
	else {
		dbConnect::raise("You must have a copy of 'PhoneControl Database Location' in the same folder as the application");
	}
	StandardFileReply	macFileReply;
	macFileReply.sfGood = true;
#else
	SFTypeList			typeList;
	short				numTypes;
	StandardFileReply	macFileReply;

	typeList[0] = kFileType;
	numTypes = 1;

	UDesktop::Deactivate();
	::StandardGetFile(nil, numTypes, typeList, &macFileReply);
	UDesktop::Activate();
#endif
	if (macFileReply.sfGood) {
		SendAEOpenDoc(macFileReply.sfFile);
	}
}

// ---------------------------------------------------------------------------
//		 ObeyCommand
// ---------------------------------------------------------------------------
//	Respond to commands

Boolean
CPhoneControlApp::ObeyCommand(
	CommandT	inCommand,
	void		*ioParam)
{
	Boolean	cmdHandled = true;

	switch (inCommand) {

case cmd_About:
CAboutDialog::CreateAboutDialog(this);
break;

	// +++ Add cases here for the commands you handle
	//		Remember to add same cases to FindCommandStatus below
	//		to enable/disable the menu items for the commands

	default:
			cmdHandled = LDocApplication::ObeyCommand(inCommand, ioParam);
		break;
	}

	return cmdHandled;
}


// ---------------------------------------------------------------------------
//		 FindCommandStatus
// ---------------------------------------------------------------------------
//	Pass back status of a (menu) command

void
CPhoneControlApp::FindCommandStatus(
	CommandT	inCommand,
	Boolean		&outEnabled,
	Boolean		&outUsesMark,
	Char16		&outMark,
	Str255		outName)
{
	outUsesMark = false;

	switch (inCommand) {

	// +++ Add cases here for the commands you handle

	default:
			LDocApplication::FindCommandStatus(inCommand, outEnabled, outUsesMark,
												outMark, outName);
		break;
	}
}
