//========================================================================================
//
//	File:				ContentView.cpp
//	Release Version:	$ ODF 1 $
//
//	Copyright:			(c) 1996 by Bowers Development Corp., all rights reserved.
//
//========================================================================================

#ifndef CONTENTVIEW_H
#include "ContentView.h"
#endif

// ----- Framework Layer -----

#ifndef FWFRAME_H
#include "FWFrame.h"
#endif

#ifndef FWSCLBAR_H
#include "FWSclBar.h"
#endif

//========================================================================================
// RunTime Info
//========================================================================================

#ifdef FW_BUILD_MAC
#pragma segment fwgadgts
#endif

FW_DEFINE_CLASS_M1(CContentView, FW_CSuperView);

const FW_ClassTypeConstant LContentView = FW_TYPE_CONSTANT('C','n','t','V');
FW_REGISTER_ARCHIVABLE_CLASS(LContentView, CContentView, CContentView::Create, FW_CView::Read, CContentView::Destroy, FW_CView::Write)

//========================================================================================
// CContentView
//========================================================================================

//----------------------------------------------------------------------------------------
// CContentView::CContentView
//----------------------------------------------------------------------------------------

CContentView::CContentView(Environment* ev, 
						FW_CSuperView* container,
						const FW_CRect& contentRect,
						const FW_CPoint& extent) :
	FW_CSuperView(ev, container, contentRect, 0, extent, FW_kXYScrolling)
{	
}

//----------------------------------------------------------------------------------------
// CContentView::CContentView
//----------------------------------------------------------------------------------------

CContentView::CContentView(Environment* ev) :
	FW_CSuperView(ev)
{	
}

//----------------------------------------------------------------------------------------
// CContentView::~CContentView
//----------------------------------------------------------------------------------------

CContentView::~CContentView()
{
}

//----------------------------------------------------------------------------------------
// CContentView::CalcExtent
//----------------------------------------------------------------------------------------

void CContentView::CalcExtent(Environment* ev)
{
	FW_Fixed	maxBottom = FW_IntToFixed (0);
	FW_Fixed	maxRight = FW_IntToFixed (0);
	FW_CPoint	subExtent;

	FW_CViewIterator ite(this);
	for (FW_CView* subview = ite.First(); ite.IsNotComplete(); subview = ite.Next())
	{
		subExtent = subview->GetLocation (ev) + subview->GetExtent (ev);
		maxBottom = FW_Maximum (maxBottom, subExtent.y);
		maxRight  = FW_Maximum (maxRight,  subExtent.x);
	}
	SetExtent (ev, FW_CPoint (maxRight, maxBottom));
}

//----------------------------------------------------------------------------------------
// CContentView::PostCreateViewFromStream
//----------------------------------------------------------------------------------------

void CContentView::PostCreateViewFromStream(Environment* ev)
{
	CalcExtent (ev);
}

//----------------------------------------------------------------------------------------
//	CContentView::Create
//----------------------------------------------------------------------------------------

void* CContentView::Create(FW_CReadableStream& stream, FW_ClassTypeConstant type)
{
FW_UNUSED(stream);
FW_UNUSED(type);
	FW_SOMEnvironment ev;
	return new CContentView(ev);
}

//----------------------------------------------------------------------------------------
//	CContentView::Destroy
//----------------------------------------------------------------------------------------

void CContentView::Destroy(void* object, FW_ClassTypeConstant type)
{
FW_UNUSED(type);
	CContentView* self = (CContentView*) object;
	delete self;
}

//----------------------------------------------------------------------------------------
//	CContentView::Flatten
//----------------------------------------------------------------------------------------

void CContentView::Flatten(Environment* ev, FW_CWritableStream& archive) const
{
	FW_CSuperView::Flatten(ev, archive);
	
	// [LSD] to complete
}

//----------------------------------------------------------------------------------------
//	CContentView::InitializeFromStream
//----------------------------------------------------------------------------------------
// This method is required to handle the custom resource fields added to the base type

void CContentView::InitializeFromStream(Environment* ev, FW_CReadableStream& stream)
{
	// Read-in the base resource first
	FW_CSuperView::InitializeFromStream(ev, stream);
}
