//========================================================================================
//
//	File:				PictView.cpp
//	Release Version:	$ ODF 1 $
//
//	Copyright:			(c) 1996 by Bowers Development Corp., all rights reserved.
//
//========================================================================================

#ifndef PICTVIEW_H
#include "PictView.h"
#endif

// ----- Framework Layer -----

#ifndef FWCONTXT_H
#include "FWContxt.h"
#endif

// ----- OS Layer -----

#ifndef FWPICSHP_H
#include "FWPicShp.h"
#endif

//========================================================================================
// RunTime Info
//========================================================================================

#ifdef FW_BUILD_MAC
#pragma segment fwgadgts
#endif

FW_DEFINE_CLASS_M1(CPictView, FW_CView)

const FW_ClassTypeConstant LPictView = FW_TYPE_CONSTANT('P','c','t','V');
FW_REGISTER_ARCHIVABLE_CLASS(LPictView, CPictView, CPictView::Create, FW_CView::Read, CPictView::Destroy, FW_CView::Write)

//========================================================================================
// CPictView
//========================================================================================

//----------------------------------------------------------------------------------------
// CPictView::CPictView
//----------------------------------------------------------------------------------------

CPictView::CPictView (
	Environment*		ev, 
	FW_CSuperView*		container,
	const FW_CRect&		contentRect,
	ODID				viewId,
	FW_ResourceId		resourceId)
	: FW_CView(ev, container, contentRect, viewId)
	, fPicture(NULL)
{
	InitPicture (ev, resourceId);
}

CPictView::CPictView(Environment* ev)
	: FW_CView(ev)
	, fPicture(NULL)
{	
}

//----------------------------------------------------------------------------------------
// CPictView::~CPictView
//----------------------------------------------------------------------------------------

CPictView::~CPictView()
{
	delete fPicture;
}

//----------------------------------------------------------------------------------------
//	CPictView::InitPicture
//----------------------------------------------------------------------------------------

void CPictView::InitPicture (
	Environment*	ev, 
	FW_ResourceId	resourceId)
{
	FW_CSharedLibraryResourceFile	resFile (ev);	
	FW_CPicture		picture (resFile, resourceId);
	FW_CRect		bounds;
	picture.GetPictBounds (bounds);
	fPicture = FW_NEW(FW_CPictureShape, (picture, bounds));
}

//----------------------------------------------------------------------------------------
// CPictView::Draw
//----------------------------------------------------------------------------------------

void CPictView::Draw(Environment *ev, ODFacet* odFacet, ODShape* invalidShape)
{
	FW_CViewContext		vc (ev, this, odFacet, invalidShape);

	fPicture->Render(vc);
}

//----------------------------------------------------------------------------------------
//	CPictView::Create
//----------------------------------------------------------------------------------------

void* CPictView::Create(FW_CReadableStream& stream, FW_ClassTypeConstant type)
{
FW_UNUSED(stream);
FW_UNUSED(type);
	FW_SOMEnvironment ev;
	return new CPictView(ev);
}

//----------------------------------------------------------------------------------------
//	CPictView::Destroy
//----------------------------------------------------------------------------------------

void CPictView::Destroy(void* object, FW_ClassTypeConstant type)
{
FW_UNUSED(type);
	CPictView* self = (CPictView*) object;
	delete self;
}

//----------------------------------------------------------------------------------------
//	CPictView::Flatten
//----------------------------------------------------------------------------------------

void CPictView::Flatten(Environment* ev, FW_CWritableStream& archive) const
{
	FW_CView::Flatten(ev, archive);
	
	// To complete: this method should stream out the picture's resource ID
	FW_DEBUG_MESSAGE("CPictView::Flatten is not completed yet");
}

//----------------------------------------------------------------------------------------
//	CPictView::InitializeFromStream
//----------------------------------------------------------------------------------------

void CPictView::InitializeFromStream(Environment* ev, FW_CReadableStream& stream)
{
	// Read-in the base resource first
	FW_CView::InitializeFromStream(ev, stream);
	
	// Read the picture's ID and create it
	FW_ResourceId	resourceId;

	stream >> resourceId;

	InitPicture (ev, resourceId);
}
