//========================================================================================
//
//	File:				RectView.cpp
//	Release Version:	$ ODF 1 $
//
//	Copyright:			(c) 1996 by Bowers Development Corp., all rights reserved.
//
//========================================================================================

#ifndef RECTVIEW_H
#include "RectView.h"
#endif

// ----- Framework Layer -----

#ifndef FWCONTXT_H
#include "FWContxt.h"
#endif

// ----- OS Layer -----

#ifndef FWRECSHP_H
#include "FWRecShp.h"
#endif

//========================================================================================
// RunTime Info
//========================================================================================

#ifdef FW_BUILD_MAC
#pragma segment fwgadgts
#endif

FW_DEFINE_CLASS_M1(CRectView, FW_CSuperView);

const FW_ClassTypeConstant LRectView = FW_TYPE_CONSTANT('R','c','t','V');
FW_REGISTER_ARCHIVABLE_CLASS(LRectView, CRectView, CRectView::Create, FW_CView::Read, CRectView::Destroy, FW_CView::Write)

//========================================================================================
// CRectView
//========================================================================================

//----------------------------------------------------------------------------------------
// CRectView::CRectView
//----------------------------------------------------------------------------------------

CRectView::CRectView(Environment* ev, 
						FW_CSuperView* container,
						const FW_CRect& contentRect,
						const FW_CPoint& extent) :
	FW_CSuperView(ev, container, contentRect, 0/* viewID */, extent, FW_kNoScrolling)
{
	// later add parameters for line width, fill pat, colors, etc.	
}

//----------------------------------------------------------------------------------------
// CRectView::CRectView
//----------------------------------------------------------------------------------------

CRectView::CRectView(Environment* ev) :
	FW_CSuperView(ev)
{	
}

//----------------------------------------------------------------------------------------
// CRectView::~CRectView
//----------------------------------------------------------------------------------------

CRectView::~CRectView()
{
}

//----------------------------------------------------------------------------------------
// CPictView::Draw
//----------------------------------------------------------------------------------------

void CRectView::Draw(Environment *ev, ODFacet* odFacet, ODShape* invalidShape)
{
	FW_CViewContext		vc (ev, this, odFacet, invalidShape);
	FW_CRect			bounds = GetBounds(ev);
	bounds.Place(FW_kFixed0, FW_kFixed0);	

	FW_CRectShape::RenderRect(vc, bounds, FW_kFrame);
}

//----------------------------------------------------------------------------------------
// CRectView::Create
//----------------------------------------------------------------------------------------

void* CRectView::Create(FW_CReadableStream& stream, FW_ClassTypeConstant type)
{
FW_UNUSED(stream);
FW_UNUSED(type);
	FW_SOMEnvironment ev;
	return new CRectView(ev);
}

//----------------------------------------------------------------------------------------
// CRectView::Destroy
//----------------------------------------------------------------------------------------

void CRectView::Destroy(void* object, FW_ClassTypeConstant type)
{
FW_UNUSED(type);
	CRectView* self = (CRectView*) object;
	delete self;
}

//----------------------------------------------------------------------------------------
// CRectView::Flatten
//----------------------------------------------------------------------------------------

void CRectView::Flatten(Environment* ev, FW_CWritableStream& archive) const
{
	FW_CSuperView::Flatten(ev, archive);
	
	// later write line width, fill pat, colors, etc.	
}

//----------------------------------------------------------------------------------------
// CRectView::InitializeFromStream
//----------------------------------------------------------------------------------------
// This method is required to handle the custom resource fields added to the base type

void CRectView::InitializeFromStream(Environment* ev, FW_CReadableStream& stream)
{
	// Read-in the base resource first
	FW_CSuperView::InitializeFromStream(ev, stream);

	// later add parameters for line width, fill pat, colors, etc.	
}
